!----------------------------------------------------------------------
! All the routines needed to give the binning in f and in r
!
!----------------------------------------------------------------------
module binning
      implicit none
      type bin_prms
         ! binning vars for f
         integer  :: n_f_lo, n_f_hi, nf
         double precision   :: f_mid, f_lo_lgint , f_hi_lgint 
         ! binning vars for r
         integer  :: n_r_lo,  n_r_hi, nr
         double precision   :: r_mid, r_hi_lgint
      end type bin_prms
      
      type(bin_prms) bn
      data   bn%n_f_lo     /100/,    bn%n_f_hi     /260/    &
           & bn%f_mid      /0.2d0/,  bn%f_lo_lgint /0.1d0/  &
           & bn%f_hi_lgint /0.05d0/                         &
           & bn%n_r_lo     /10/,     bn%n_r_hi     /150/    &
           & bn%r_mid      /1.0d0/,  bn%r_hi_lgint /0.1d0/  

end module binning


!----------------------------------------------------------------------
! Interface blocks for the binning and associated routines
!----------------------------------------------------------------------
module bin_interface
      interface
         function bin_of_f(f)
               implicit none
               double precision, intent(in) :: f
               integer            :: bin_of_f
         end function bin_of_f
      end interface

      interface
         function f_of_bin(bin)
               implicit none
               integer, intent(in) :: bin
               double precision              :: f_of_bin
         end function f_of_bin
      end interface

      interface
         function irad(r)
               use binning
               implicit none
               integer                   :: irad
               double precision, intent(in)        :: r
         end function irad
      end interface
      
      interface
         function rad_lo(i)
               use binning
               implicit none
               double precision                    :: rad_lo
               integer, intent(in)       :: i
         end function rad_lo
      end interface

      interface
         function rad_rnd(i)
               implicit none
               double precision              :: rad_rnd
               integer, intent(in) :: i
         end function rad_rnd
      end interface

      interface
         subroutine allc_read_fbins(n_y, n_prev_events)
               implicit none
               integer, intent(in)           :: n_y
               integer, intent(in), optional :: n_prev_events
         end subroutine allc_read_fbins
      end interface
      
      interface
         subroutine write_fbins(n_events, iufdt_arg)
               implicit none
               integer, intent(in)           :: n_events
               integer, intent(in), optional :: iufdt_arg
         end subroutine write_fbins
      end interface


      
      interface
         subroutine add_f_to_bins(ext_dips1, n1, ext_dips2, n2, fbins_lcl)
               use gluons
               implicit none
               !----------------------------------------------------------
               type(dpl_pntr), dimension(:), intent(in) :: ext_dips1,&
                    & ext_dips2
               integer,            intent(in)    :: n1, n2
               double precision,   intent(inout) :: fbins_lcl( -1:,0:)
         end subroutine add_f_to_bins
      end interface
      

end module bin_interface

   

!----------------------------------------------------------------------
! Given a value for f, return a bin number. Not optimised (shouldn't be
! needed, since only called at end
!
! GPS 11/8/95 (CCN7 25)
!----------------------------------------------------------------------
function bin_of_f(f)
      use binning
      implicit none
      double precision, intent(in) :: f
      integer            :: bin_of_f
      double precision             :: logf

      !----------------------------------------------------------------
      if (f <= 0d0) then
         bin_of_f = -1
         return
      end if
      
      bin_of_f = bn%n_f_lo

      logf = log(f / bn%f_mid)
      if (logf < 0d0) then
         bin_of_f = bin_of_f + floor(logf / bn%f_lo_lgint)
         bin_of_f = max(-1, bin_of_f) ! everything below this counts as zero
      else
         bin_of_f = bin_of_f + floor(logf / bn%f_hi_lgint)
         bin_of_f = min(bin_of_f, bn%n_f_lo + bn%n_f_hi)
      end if
end function bin_of_f


!----------------------------------------------------------------------
! Given a bin number, return a value for f. If bin number is -1 then
! return zero (i.e. less than smallest bin). If it is n_f_lo+n_f_hi, then
! it is not yet clear what should be returned -- for time being don't
! treat it differently
!
! GPS 11/8/95 (CCN7 25)
!----------------------------------------------------------------------
function f_of_bin(bin)
      use binning
      implicit none
      integer, intent(in) :: bin
      double precision              :: f_of_bin

      !----------------------------------------------------------------
      if (bin == -1) then
         f_of_bin = 0d0
      elseif (bin < bn%n_f_lo) then
         f_of_bin = bn%f_mid * exp((bin - bn%n_f_lo + 0.5d0)*bn%f_lo_lgint)
      elseif (bin == bn%n_f_lo + bn%n_f_hi) then 
         ! contains all cases of f too large -- for now treat no
         ! differently 
         f_of_bin = bn%f_mid * exp((bin - bn%n_f_lo + 0.5d0)*bn%f_hi_lgint)
      else
         f_of_bin = bn%f_mid * exp((bin - bn%n_f_lo + 0.5d0)*bn%f_hi_lgint)
      end if
end function f_of_bin



!----------------------------------------------------------------------
! Return the bin number for a given radius -- linear for r < bn%r_mid and
! logarithmic for r > bn%r_mid. Doesn't Do any extra processing beyond that -
! - so if it is given a negative radius, it will return a negative number.
! If it is given a radius larger than the max available r, it just returns
! an appropriately large bin number. The calling routine must do its own
! checking. The number returned is rounded down.
!
! GPS 27/7/95 (CCN6 185) + 12/8/95 (CCN7 27)
!----------------------------------------------------------------------
function irad(r)
      use binning
      implicit none
      integer                   :: irad
      double precision, intent(in)        :: r

      if (r < bn%r_mid) then
         irad = floor(r / bn%r_mid * bn%n_r_lo)
      else
         irad = bn%n_r_lo + floor(log(r / bn%r_mid) / bn%r_hi_lgint)
      end if
end function irad


!----------------------------------------------------------------------
! According to the procedure, return the lower edge for the given bin
! number. 
!
! GPS 12/8/95 (CCN7 27)
!----------------------------------------------------------------------
function rad_lo(i)
      use binning
      implicit none
      double precision                    :: rad_lo
      integer, intent(in)       :: i

      if (i < bn%n_r_lo) then
         rad_lo = i * bn%r_mid / bn%n_r_lo
      else
         rad_lo = bn%r_mid * exp(bn%r_hi_lgint * (i - bn%n_r_lo))
      end if
end function rad_lo


!----------------------------------------------------------------------
! Return a randomly distributed vaule for r within the given bin -- the
! distribution goes as r, in accord with the circumference of a ring
!
! GPS 12/8/95 (CCN7 27)
!----------------------------------------------------------------------
function rad_rnd(i)
      implicit none
      double precision              :: rad_rnd
      integer, intent(in) :: i
      !--- interface problems mean this has to go here ----------------
      double precision, external    :: rad_lo
      !----------------------------------------------------------------
      double precision              :: r_lo, r_hi
      double precision              :: hwrgen
      integer             :: j

      if (i < 0) then
         write(0,*) "rad_rnd shoul only be used with positive arguments"
         write(0,*) "Error: "
      end if
      
      r_lo = rad_lo(i)
      r_hi = rad_lo(i + 1)
      rad_rnd = sqrt((r_hi**2 - r_lo**2) * hwrgen(j) + r_lo**2)
end function rad_rnd


!----------------------------------------------------------------------
!----------------------------------------------------------------------
! Routines associated with reading and writing of bins, as well as
! adding to their contents (for amplitude)
!
! GPS 12/11/95 (CCN8 31)
!----------------------------------------------------------------------
!----------------------------------------------------------------------

!----------------------------------------------------------------------
! This needs to be allocatable so that the reading routine can also
! allocate it. Also, by making it global, it is easier to ensure that the
! corrects bounds (i.e. lower bounds) are used
!
! GPS 12/11/95 (CCN8 31)
!----------------------------------------------------------------------
module fbins_mdl
      implicit none
      double precision, allocatable :: fbins(:,:,:)
end module fbins_mdl

!----------------------------------------------------------------------
! The third index of the array is nominally the number of y values at
! which the amplitude is being studied -- this needs to be known so as to
! allocate an array of the right size (and to be able to read in the
! correct amount of data)
!
! n_prev_events needs to be known so as to normalise the data correctly
! and also to knwn whether to initialise fbins rather than read them in.
! In the case of reading data for analysis, no normalisation is needed.
!
! GPS 12/11/95 (CCN7 31)
!----------------------------------------------------------------------
subroutine allc_read_fbins(n_y, n_prev_events)
      use iomodule
      use sub_defs
      use binning
      use fbins_mdl
      implicit none
      integer, intent(in)           :: n_y
      integer, intent(in), optional :: n_prev_events
      !----------------------------------------------------------------
      real, allocatable :: fbins_rl(:,:)
      integer           :: i, npe_lcl

      allocate (fbins(-1:bn%nf, 0:bn%nr, n_y)) 
      write(0,*) 'Have allocated bins'

      if (present(n_prev_events)) then
         npe_lcl = n_prev_events
      else
         npe_lcl = 1
      end if

      if (npe_lcl == 0) then
         fbins = 0d0
      else
         allocate (fbins_rl(-1:bn%nf, 0:bn%nr)) 
         do i = 1, n_y
            call lcl_read_2d(iufdt, fbins_rl)
            fbins(:,:,i) = dble(fbins_rl(:,:))
         end do
	 fbins(:,:,:) = fbins(:,:,:) * npe_lcl
         deallocate (fbins_rl)
      endif
end subroutine allc_read_fbins


!----------------------------------------------------------------------
! Write the data in the correct format (i.e. as reals, not double
! precision) and normalised correctly -- better to do normalisation here
! (rather than in calling routine) to reduce the size of the array which
! needs to be duplicated
!
! GPS 12/11/95 (CCN8 33)
!----------------------------------------------------------------------
subroutine write_fbins(n_events, iufdt_arg)
      use sub_defs
      use binning
      use fbins_mdl
      implicit none
      integer, intent(in)           :: n_events
      integer, intent(in), optional :: iufdt_arg
      !------------------------------------------------------------
      integer i, j, iufdt_lcl, bnds(3), i_y_hi, i_y_lo

      if (present(iufdt_arg)) then
         iufdt_lcl = iufdt_arg
      else
         call get_io_devs(i, j, iufdt_lcl)
      end if
      
      bnds = lbound(fbins) ; i_y_lo = bnds(3)
      bnds = ubound(fbins) ; i_y_hi = bnds(3)

      do i = i_y_lo, i_y_hi
         call lcl_write_2d(iufdt_lcl, real(fbins(:,:,i)/n_events))
      end do                              
end subroutine write_fbins



!*************************************************************************
! Bin the interaction along a radius going vertically up in impact
! parameter: for each pair of dipoles forming an interaction region,
! determine whether the region lies on this radius. If it does then
! determine its contribution to the interaction at each of the points on
! the radius where samples are being taken. At the end, for each point on
! the radius, bin the amplitude.
!
! The points on the radii are randomly chosen within each bin, having a
! distribution weighted with r.
!
! The interaction regions have an extent which is twice the sum of the
! sizes of the two dipoles.
!
! Normalisation is left till just before the final binning of the
! amplitudes. 
!
! GPS 7+12/8/95 (CCN7 13 + 29)
! based on routine from spread.f90 (CCN3 155-7)
!*************************************************************************

subroutine add_f_to_bins(ext_dips1, n1, ext_dips2, n2, fbins_lcl)
      use bin_interface, except => add_f_to_bins
      use gluons
      use constants
      use binning
      implicit none
      !----------------------------------------------------------------
      ! Arguments
      type(dpl_pntr), dimension(:), intent(in)    :: ext_dips1, ext_dips2
      integer,                      intent(in)    :: n1, n2
      double precision,             intent(inout) :: fbins_lcl(-1:,0:)
      !---- defines interaction parameters for a pair of dipoles ------
      type intrctn
	 complex(kind(1d0))  :: psn
	 double precision    :: r, sigma
      end type intrctn
      !---- Local variables -------------------------------------------
      integer,        parameter                 :: grid_size = 99
      type(intrctn)                             :: region
      integer      i1, i2, i, j      ! counters
      complex(kind(1d0))   dipole_1(1:2), dipole_2(1:2)
      double precision       h_rgn_lo, h_rgn_hi 
      integer      iv_rgn_lo, iv_rgn_hi
      !
      !---- specifically for getting interaction at different r -----
      double precision     intrn(0:bn%nr)
      complex(kind(1d0)) rpsns(0:bn%nr)

      !--------------------------------------------------------------------
      ! Set up the (random) positions in each bin that will be used for
      ! this run.
      do i = 0, bn%n_r_lo + bn%n_r_hi
         rpsns(i) = cmplx(0d0, rad_rnd(i))
      end do
      intrn = 0d0

      do i1 = 1, n1
	 do i2 = 1, n2
	    i = i1 + n1*(i2-1)   
	    ! Work out position and size of the region
	    ! vector to add to 2nd dipole such that its centre is at the
	    ! same point as the centre of the first dipole
            region%psn = (ext_dips1(i1)%p%hi_y_gluon%psn + &
                                   ext_dips1(i1)%p%lo_y_gluon%psn - &
                                   ext_dips2(i2)%p%hi_y_gluon%psn - &
                                   ext_dips2(i2)%p%lo_y_gluon%psn)/2.0d0

	    !----- The extent of the interaction region -------------------
            region%r = (ext_dips1(i1)%p%size + ext_dips2(i2)%p%size) * 2d0

            !----- points are taken along a line vertically up only ------
	    h_rgn_lo =  real(region%psn) - region%r
	    h_rgn_hi =  real(region%psn) + region%r
            if (h_rgn_lo > 0d0 .or. h_rgn_hi < 0d0) cycle

            !----- get bin on vertical line ------------------------------
            iv_rgn_lo = irad(aimag(region%psn) - region%r)
            iv_rgn_hi = irad(aimag(region%psn) + region%r)
            !----- the first condition should never occur...? -----------
            if (iv_rgn_lo > bn%nr .or. iv_rgn_hi < 0) cycle
	    
	    ! set up dipole positions in easily usable form (may waste some
	    ! time: but neater to program -- come back to it later?)
	    dipole_1(1) = ext_dips1(i1)%p%hi_y_gluon%psn
	    dipole_1(2) = ext_dips1(i1)%p%lo_y_gluon%psn
	    dipole_2(1) = ext_dips2(i2)%p%hi_y_gluon%psn
	    dipole_2(2) = ext_dips2(i2)%p%lo_y_gluon%psn

            iv_rgn_lo = max(0, iv_rgn_lo)
            iv_rgn_hi = min(bn%nr, iv_rgn_hi)


	    ! Add contributions from this interaction region to all
	    ! relevant radial points 
	    do j = iv_rgn_lo, iv_rgn_hi
               intrn(j) = intrn(j) + potential(dipole_1, dipole_2 + rpsns(j))
	    end do
	 end do
      end do

      ! Put in magnitude of interaction into grid
      intrn   = intrn * alpha_no_nc**2 / 2d0

      ! For each r, bin the interaction
      do i = 0, bn%nr
         j = bin_of_f(intrn(i))
         fbins_lcl(j,i) = fbins_lcl(j,i) + 1d0
      end do

contains

!***************************************************************************
! Routine to work out the potential^2 between two dipoles with sources
! positioned at positions dipole_1 and dipole_2 -- without normalisation
! of (alpha_no_nc**2/ 2) -- NB interaction by two gluon exchange is 
! proportional to square of dipole dipole potential
!
! Equation # in write up?
!
! GPS 16/5/95 (CCN5 65)
!***************************************************************************
function potential(dipole_1, dipole_2)
     implicit none
     double precision      potential
     complex(kind(1d0)), intent(in)  :: dipole_1(1:2), dipole_2(1:2)
     double precision      numerator, denominator
     double precision,     parameter   :: max_potn = 1d3
     
     ! Work out square of absolute values of each quantity -- hopefully
     ! this is the fastest way of dealing with things. Requires factor of 4
     ! after the log though. 
     numerator    = abs_sqrd(dipole_2(2) - dipole_1(2)) * &
                    abs_sqrd(dipole_2(1) - dipole_1(1)) 
     denominator  = abs_sqrd(dipole_2(2) - dipole_1(1)) * &
                    abs_sqrd(dipole_2(1) - dipole_1(2))
     if (numerator == 0d0 .or. denominator == 0d0) then
        potential = max_potn
     else
        ! remember factor of 4 to take into account no sqrt in ratios
        potential = (log(numerator / denominator))**2 / 4d0 
     end if
     potential = min(potential, max_potn)
end function potential

! needed many times and much quicker than doing (abs())**2
function abs_sqrd(vector)
      double precision abs_sqrd
      complex(kind(1d0)), intent(in) :: vector
      
      abs_sqrd = real(vector)**2 + aimag(vector)**2
end function abs_sqrd
          

end subroutine add_f_to_bins




