!----------------------------------------------------------------------
! Just a simple routine to save the user having to zero the number of
! dipoles before calling extract_dipoles
!
! GPS 7/11/95 (CCN8 21)
!----------------------------------------------------------------------
subroutine extract_onium_dpls(onium, y, extracted_dipoles, n)
      use sub_defs, except => extract_onium_dpls
      use gluons 
      implicit none 
      type(dipole),     intent(in), target        :: onium 
      double precision, intent(in)                :: y 
      type(dpl_pntr), dimension(:), intent(inout) :: extracted_dipoles 
      integer,        intent(inout)               :: n 
      !------------------------------------------------------------

      n = 0
      call extract_dipoles(onium, y, extracted_dipoles, n)
end subroutine extract_onium_dpls

!---------------------------------------------------------------------- 
!     Recursively extract any dipoles that either have no children      
!     or whose children have rapidities >= y.                           
!     INTERFACE BLOCK: sub_defs.f, module sub_defs                      
!
!     Dipoles are put into the array extracted_dipoles, at the next
!     available position (obtained by incrementing n)
!
! GPS 16/11/94 (CCN3 47)                                                
!---------------------------------------------------------------------- 
recursive subroutine extract_dipoles(top,y,extracted_dipoles,n) 
      use gluons 
      implicit none 
      type(dipole),     intent(in), target        :: top 
      double precision, intent(in)                :: y 
      type(dpl_pntr), dimension(:), intent(inout) :: extracted_dipoles 
      integer,        intent(inout)               :: n 
                                                                        
      if (associated(top%child_1)) then 
         if (top%child_1%hi_y_gluon%rapidity <= y) then 
            call extract_dipoles(top%child_1,y,extracted_dipoles,n) 
            call extract_dipoles(top%child_2,y,extracted_dipoles,n) 
            return 
         endif 
      endif 
                                                                        
      n = n + 1 
      extracted_dipoles(n)%p => top 
                                                                        
end subroutine extract_dipoles

!-------------------------------------------------------------------------
! Set of routines designed to provide a simpler interface to the evolution
! and extraction routines
!------------------------------------------------------------------------
! Reset the various counters associated with storage of dipoles and gluons
! -- should generally be called before a set of onia are evolved (i.e.
! once round each loop of a program
!
! GPS 7/11/95 
!------------------------------------------------------------------------
subroutine reset_counters
      use evoln_intrfc
      implicit none

      dipole_n = 0
      gluon_n = 0
end subroutine reset_counters

!-----------------------------------------------------------------------
! Return a pointer to an onium with random orientation and size onm_size
! Just a "convenience" routine to save extra code each time this is 
! needed
!
! GPS 7/11/95 (CCN8 20)
!-----------------------------------------------------------------------
subroutine init_onium_rnd_phi(onium, onm_size)
      use sub_defs, except => init_onium_rnd_phi
      use gluons
      use constants
      implicit none
      type(dipole),     pointer     :: onium
      double precision, intent(in)  :: onm_size
      !-------------------------------------------------------------------
      double precision  phi, hwrgen
      integer           i

      phi = 2d0 * pi * hwrgen(i)
      call init_onium(onium, onm_size, phi)
end subroutine init_onium_rnd_phi



!----------------------------------------------------------------------
! ROUTINES FOR BACKWARDS COMPATABILITY
!----------------------------------------------------------------------

!----------------------------------------------------------------------
! Evolution routine that takes orinigal args and returns them as they
! would have been treated in previous version of the prog
!
! GPS 7/11/95
!----------------------------------------------------------------------
subroutine evolve(parent, maxy, lcl_dipole_n, lcl_gluon_n)
      use gluons
      use evoln_intrfc
      use emergency
      implicit none
      !------------------------------------------------------------
      type(dipole), intent(inout)  ::  parent 
      double precision, intent(in) ::  maxy 
      integer, intent(inout)       ::  lcl_dipole_n, lcl_gluon_n 
      !------------------------------------------------------------
      integer  :: dpl_index

      dipole_n = lcl_dipole_n
      gluon_n  = lcl_gluon_n

      call evolve_onium(parent, maxy, dpl_index)

      if (gluon_n == lcl_gluon_n) then
         ! either dynamic allocation -- in which case gluon_n and dipole_n
         ! won't have been changed -- gluon_n needs to be incremented by
         ! number of new gluons
         ! Or there was no evolution, in which case the following lines
         ! will not change anything anyway
         gluon_n = lcl_gluon_n + (dpl_index - 1)/2
      end if

      !-- calling routine needs to think that lcl_* are the actual index
      ! variables 
      lcl_dipole_n = dipole_n
      lcl_gluon_n  = gluon_n
end subroutine evolve

!----------------------------------------------------------------------
! The next two routines ensure that the gluon and dipole numbering are
! passed on as they would have been in the original routines
!
! GPS 7/11/95
!----------------------------------------------------------------------
subroutine init_rnd_phi(dpl_sz, lcl_gluon_n, lcl_dipole_n) 
      use sub_defs
      use gluons
      use evoln_intrfc
      implicit none
      double precision, intent(in)    :: dpl_sz
      integer,          intent(inout) :: lcl_gluon_n, lcl_dipole_n
      !------------------------------------------------------------
      type(dipole),     pointer       :: onium

      gluon_n  = lcl_gluon_n
      dipole_n = lcl_dipole_n
      onium => dipole_set(1)  ! dummy line to avoid a warning
      call init_onium_rnd_phi(onium, dpl_sz)
      lcl_gluon_n  = gluon_n
      lcl_dipole_n = dipole_n
end subroutine init_rnd_phi

!----------------------------------------------------------------------
! To produce a dipole oriented with angle = 0
subroutine init_dipole(dpl_sz, lcl_gluon_n, lcl_dipole_n) 
      use sub_defs
      use gluons
      use evoln_intrfc
      implicit none
      double precision, intent(in)    :: dpl_sz
      integer,          intent(inout) :: lcl_gluon_n, lcl_dipole_n
      !------------------------------------------------------------
      type(dipole),     pointer       :: onium

      gluon_n  = lcl_gluon_n
      dipole_n = lcl_dipole_n
      onium => dipole_set(1)  ! dummy line to avoid a warning
      call init_onium(onium, dpl_sz, 0d0)
      lcl_gluon_n  = gluon_n
      lcl_dipole_n = dipole_n
end subroutine init_dipole









        
      
