!**********************************************************************
! All the routines associated with generating a random transverse position
! for a gluon
!**********************************************************************


                                                                        
!---------------------------------------------------------------------- 
! Generate random values for b, the distance of a gluon from one of the
! sources of a dipole, and phi the angle relative to the line between the
! two sources. Resulting position is restricted to be in one quadrant of
! the available region.
! phi is generated without worrying about any upper cutoff -- the
! combination b,phi is then rejected if the distance from either of the
! sources is larger than the upper cutoff (assuming that it is
! implemented). 
!                                                                       
! GPS 15/11/94 CCN3(37-45)
! GPS 22/11/94 CCN3(75) Remember to call p_true without a hi_cut
!---------------------------------------------------------------------- 
subroutine gnrt_pnt(a,b,phi,weight) 
      use constants 
      use cuts 
      implicit none
      double precision, intent(in)   ::  a
      double precision, intent(out)  ::  b, phi, weight

      double precision x,x_lo,x_hi,p,p_hi  
      double precision k_sqrd_inv,acptnc_fctr,ka2 
! ? acptnc fctr = 3/(4+k**2)
      parameter (k_sqrd_inv = 1.0d0, acptnc_fctr = 0.6d0) 
      double precision dneg 
      parameter (dneg=-1.0d0) 
      double precision hwrgen,p_true,p_fake,phi_sym 
      integer i 
      data i/0/ 
                                                                        
      ka2 = a**2 * k_sqrd_inv 
                                                                        
      ! get range needed for the uniform random variable
      if (hi_cut_implem) then 
         x_hi = 0.5d0*log(r_cut_hi**2/(ka2+r_cut_hi**2)) 
      else 
         x_hi = 0.0d0 
      endif 
      x_lo = 0.5d0*log(r_cut_lo**2/(ka2+r_cut_lo**2)) 
                                                                        
      do
         ! generate b with approximately the right distribution
         ! (correct asymptotically)
         x = hwrgen(i) 
         b = sqrt(ka2/(exp(-2.0d0*(x*(x_hi-x_lo)+x_lo))-1.0d0)) 
                                                                        
         ! the approximate distribution being used
         p_fake = 2*pi/(b*(1+b**2/ka2)) 
         x = hwrgen(i) 
         ibtot = ibtot + 1
         ! conditionally reject it to get exactly the right distribution
         ! accptnc_fctr is chosen (see above) to be the smallest value of 
         ! p_fake/p_true (usually occurs for b = a/2 where there is a
         ! cusp). 
         if (x .gt. acptnc_fctr*p_true(a,b,dneg,.false.)/p_fake) then
            ibrej = ibrej + 1 ! internal stats
            cycle
         endif
         ! all points now have the same weight
         weight = 1.0d0 
         
         ! generate an angle
         p = hwrgen(i) 
         if (a .ge. 2.0d0*b) then 
            ! whole range of angles is available
            p_hi = pi/2.0d0 
            phi = pi - 2*atan((a+b)/(a-b)*tan(p*p_hi)) 
         else 
            ! only a limited range of angles is available (to stay in the
            ! correct quadrant)
            phi_sym = pi - acos (a/(2.0*b)) 
            if (abs(a-b)/a .gt. 1.0d-6) then 
               p_hi = atan((a-b)/(a+b)*tan(phi_sym/2.0d0)) 
               phi = pi - 2*atan((a+b)/(a-b)*tan(p*p_hi)) 
            else 
               ! use asymptotic limit if a is too close to b
               p_hi = tan(phi_sym/2.0d0) 
               phi = pi - 2*atan(p*p_hi) 
            endif 
         endif 
         
!         if ((a**2+b**2-2*a*b*cos(phi)) .le. 0) write(6,*) phi ! debug       
         iangtot = iangtot + 1  ! internal statistics

         ! make sure distance frmo other source is not larger than 
         ! r_cut_hi
         if (hi_cut_implem .and.                                           &
     &        sqrt(a**2+b**2-2*a*b*cos(phi)) .ge. r_cut_hi) then
            iangrej = iangrej + 1
            cycle
         endif
            
         exit
      end do
end subroutine gnrt_pnt

!====================================================================== 
!     Given the positions of the two parent sources, and the b and phi  
!     as returned by the gnrt_pnt routine, return a position for the 
!     emitted gluon -- i.e. choose a quadrant for the gluon   
!                                                                       
! GPS 9/11/94 (CCN3 p.20-25)                                            
!====================================================================== 
subroutine get_glu_psn(z1,z2,adz,z_glu,b,phi) 
      use cuts 
      implicit none 
      complex(kind(1d0)) z1,z2,z_glu,dz,exp_iphi 
      double precision b,phi,adz,hwrgen 
      integer i,j 
                                                                        
      dz  = z2 - z1 
      ! adz is just size of parent dipole -- which is stored in its dipole
      ! structure. 
!      adz = abs(dz)                                                    
                                                                        
      i = 4*hwrgen(j) 
      if (i .ge. 2) then 
         exp_iphi = cmplx(cos(phi),sin(phi)) 
      else 
         exp_iphi = cmplx(cos(phi),-sin(phi)) 
      endif 
                                                                        
      if (mod(i,2) .eq. 0) then 
         z_glu = z1 + exp_iphi*b*dz/adz 
      else 
         z_glu = z2 - exp_iphi*b*dz/adz 
      endif 

!DEBUG
!      print *, '---------'
!      print *, dz, abs(dz), adz
!      print *, z1, z2
!      print *, b, phi
!      print *, z_glu, i
!      print *, abs(z_glu-z1), abs(z_glu-z2)
end subroutine get_glu_psn


!---------------------------------------------------------------------- 
!     Routine to return a value proportional to the probability         
!     of emitting a gluon a distance r from source 1, in the right      
!     hand half of the emission plane.                                  
!     If r_cut_hi <= 0 then no high cut is implemented. Otherwise, it is.
!     Could optimise by removing some operations at, as long as do the same
!     in all other relevant places as well.                             
!
! Result is actually just the value of the angular integral of the function
! a^2 / (a - r)^2 where the angle of r wrt the dipole axis is varied
!        -   -   
! over the whole range available while r stays in the half plane
! associated with one of the sources (i.e. with division of plane
! perpendicular to dipole axis) -- 
!
! GPS 5/11/94    CCN3(5)                                                
!     22/11/94 (CCN3 75: calling arguments changed to allow calling
!                        routine to specify upper cut details)
!     Routine based on earlier routine from Dipole_DE:                  
! [GPS 29/9/94    CCN2(99-103, 115-117)]                                
! [    4/10/94    CCN2(135)            ]                                
!---------------------------------------------------------------------- 
function p_true(a, r, r_cut_hi, hi_cut_implem) 
      use constants 
      implicit none 
      double precision, intent(in)   :: a, r, r_cut_hi
      logical, intent(in)  :: hi_cut_implem
      double precision p_true, num_sub
      double precision phi_sym, phi_hi, tan_sym, tan_hi 
      logical hi_cut, sym_cut
                                                                        
! Commented out for speed 
!#ifdef DEBUG                                                           
!      if (hi_cut_implem .and. r_cut_hi .lt. a) then                    
!         write (0,*) "ERROR in function p_true (v. 2cuts):"            
!         write (0,*) "Configuration where hi cutoff"                   
!         write (0,*) "is less than emitting dipole size is "           
!         write (0,*) "not yet supported"                               
!         stop                                                          
!      endif                                                            
!                                                                       
!      if ((r .ge. r_cut_hi) .and. hi_cut_implem) then                  
!         p_true = 0.0d0                                                
!         return                                                        
!      endif                                                            
!#endif                                                                 
                                                                        
      ! Determine which cutoffs are relevant
      if (hi_cut_implem .and. r+a .gt. r_cut_hi) then 
         ! upper cutoff limits angular integral at phi_hi
         phi_hi = pi - acos ((a**2+r**2-r_cut_hi**2)/(2*a*r)) 
         tan_hi = tan(phi_hi/2) 
         hi_cut = .true. 
      else 
         ! upper cutoff not relevant here
         hi_cut = .false. 
      endif 
                                                                        
      if (2.0d0*r .gt. a) then 
         ! angular integral gets cut off at edge of half plane
         phi_sym = pi - acos (a/(2.0*r)) 
         tan_sym = tan(phi_sym/2) 
         sym_cut = .true. 
      else 
         ! all region is in this half plane
         sym_cut = .false. 
      endif 
                                                                        
                                                                        
      ! Determine number produced without any higher cutoff
      if (sym_cut) then 
         if (abs(a-r)/a .gt. 1.0d-6) then 
            p_true = 4/(a**2-r**2) * atan((a-r)/(a+r) * tan_sym)
         else 
            p_true=tan_sym/a**2 
         endif 
      else 
         p_true = 2*pi/abs(a**2-r**2) 
      endif 
                                                                        
                                                                        
      ! Calculate amount to subtract for higher cutoff
      if (hi_cut) then 
         if (abs(a-r)*tan_hi/a .gt. 1.0d-6) then 
            num_sub = 4/(a**2-r**2) * atan((a-r)/(a+r) * tan_hi)
         else 
            num_sub=tan_hi/a**2 
         endif 
      else 
         num_sub = 0.0d0 
      endif 
                                                                        
      ! make sure that some of the region is available
      ! take into account measure due to use of polar coords
      p_true = p_true - num_sub 
      if (p_true .gt. 0.0d0) then 
         p_true = a**2*p_true/r 
      else 
         p_true = 0.0d0 
      endif 
                                                                        
end function p_true

