!********************************************************************** 
! Modules for the code below                                            
!********************************************************************** 
module life_intgn 
      implicit none 
      double precision a_cmn 

      ! structure to store the lifetimes for a number of dipole sizes in
      ! the range lo to hi -- for quick retrieval of lifetime at a later
      ! stage.
      type store 
         double precision, dimension(:), pointer  :: contents 
         double precision  lo, hi, interval 
         integer n_intervals 
      end type 
              
      ! break up dipoles sizes into three regions.
      type(store) bottom, middle, top 
      ! useful constants
      double precision rec_life_const, rec_life_const_fpi 
                                                                        
end module life_intgn
                                                                        
                                                                        
!********************************************************************** 
!     Subroutine which sets up the storage of life times and            
!     associated constants. Reciprocal lifetimes are stored because     
!     they follow a straighter curve and so more effectively            
!     reproduce the intended value.                                     
!                                                                       
!     Set up values in module for the factors in front of the log       
!     when estimating the lifetime analytically -- saves                
!     multiplications.                                                  
!                                                                       
! GPS 18/11/94 (CCN3 63)                                                
!********************************************************************** 
      subroutine init_lives 
      use constants 
      use life_intgn 
      use cuts 
      implicit none 
!      double precision , parameter :: drop = 0.9d0               
      integer nrl 
                                                                        
      rec_life_const = 2d0 * pi**2/(alpha_s * nc) 
      rec_life_const_fpi = rec_life_const / (4.0d0*pi) 
      hi_cut_implem = (r_cut_hi > r_cut_lo) 
                                                                        
      bottom%lo   = r_cut_lo 
      bottom%hi   = 128.0d0*r_cut_lo 
      if (hi_cut_implem) then 
         bottom%hi = min(bottom%hi, sqrt(r_cut_lo*r_cut_hi)) 
      endif 
      
      ! Points fall on factors of two (unless above if is executed)
      bottom%n_intervals = 7*20 
      bottom%interval    = log(bottom%hi/bottom%lo)/bottom%n_intervals 
      call fill_contents(bottom) 
                                                                        
                                                                        
      if (hi_cut_implem) then 
         middle%lo = max(r_cut_hi/100.0d0, bottom%hi) 
         nrl = 40 * int(log(r_cut_hi/r_cut_lo)) 
         nrl = max(10, nrl) 
         top%lo = max(sqrt(r_cut_hi*middle%lo), r_cut_hi - nrl*r_cut_lo) 
         top%hi = r_cut_hi 
         top%n_intervals = min(1000,max(200,nrl)) 
         top%interval = log(top%hi/top%lo)/top%n_intervals 
                                                                        
         middle%hi = top%lo 
         middle%n_intervals = 200 
         middle%interval  = log(middle%hi/middle%lo)/middle%n_intervals 
                                                                        
         call fill_contents(top) 
         call fill_contents(middle) 
      endif 
                                                                        
contains 
!---------------------------------------------------------------------- 
! Save repetitive code : allocate storage and fill it up                
!---------------------------------------------------------------------- 
subroutine fill_contents(str) 
      type(store), intent(inout)  ::  str 
      ! Local variables                                                 
      double precision lifetime, a 
      integer i 
                                                                        
      allocate(str%contents(0:str%n_intervals)) 
                                                                        
      do i = 1, str%n_intervals - 1 
         a = exp(i*str%interval) * str%lo 
         str%contents(i) = lifetime(a) 
      enddo 

      ! Being on edge of limits can cause problems -- avoid them.
      ! Close to lower edge    
      a = str%lo*(1+20.0d0*epsilon(str%hi)) 
      str%contents(0) = lifetime(a) 
      ! Close to upper edge    
      a = str%hi*(1-20.0d0*epsilon(str%hi)) 
      str%contents(str%n_intervals) = lifetime(a) 
                                                                        
      ! normalise all       
      str%contents = str%contents/rec_life_const 
end subroutine fill_contents
!---------------------------------------------------------------------- 
end subroutine init_lives
      
                                                                        
                                                                        
                                                                        
!********************************************************************** 
!     Subroutine to return approximate value for lifetime of dipole     
!     based on values stored in the various matrices calculated above   
!     and a reasonable analytic approximation where the effects of the  
!     cutoffs are negligible                                            
!                                                                       
! GPS 21/11/94 CCN3(63)                                                 
!********************************************************************** 
double precision function get_life(a) 
      use life_intgn 
      use cuts 
      implicit none 
      double precision, intent(in) :: a 

                                                                        
      if (a < bottom%hi) then 
         get_life = get_from_str(bottom) 
      elseif (hi_cut_implem .and. a > top%lo ) then 
         get_life = get_from_str(top) 
      elseif (hi_cut_implem .and. a > middle%lo) then 
         get_life = get_from_str(middle) 
      else 
         ! otherwise this approximation is good to O((r_c_l/a)**2)
         get_life = rec_life_const_fpi / log(a/r_cut_lo) 
      endif 
                                                                        
contains 
!---------------------------------------------------------------------- 
!    Retrieve the lifetime from the relevant store                      
!---------------------------------------------------------------------- 
double precision function get_from_str(str) 
      type(store), intent(in)  ::  str 
      double precision la 
      integer ila 

                    
      la = log(a/str%lo)/str%interval 
      ila = int(la) 
      get_from_str = (str%contents(ila+1) - str%contents(ila))*(la - ila)&
     &               + str%contents(ila)                             
      get_from_str = 1.0d0 / get_from_str 
                                                                        
end function get_from_str
!---------------------------------------------------------------------- 
end function get_life
                                                                        
                                                                        
                                                                        
!********************************************************************** 
!     Function which returns reasonably accurate evaluation of          
!     total virtual corrections for a dipole of size a.                 
!     This is done by splitting the integration into 3 or 4 regions     
!     and dealing with each one individually -- the aim is to make      
!     make each region smooth, and not too peaked at one end -- where
!     geometrical or physical cuts come into play are points to watch   
!     out for.                                                          
!
!     Actually returns rec_life_const / lifetime.
!                                                                       
! GPS 17/11/94 CCN3(53-57)                                              
!********************************************************************** 
function lifetime(a) 
      use life_intgn 
      use cuts 
      implicit none 
      interface 
         double precision function my_dgquad(fun,a,b,i) 
         double precision, external :: fun 
         double precision  a,b 
         integer i 
         end function my_dgquad 
      end interface 
      double precision                     lifetime 
      double precision, intent(in)      :: a 
! Local variables                                                       
      double precision                  :: t_lo, t_hi
      double precision, external        :: intgrt_to_infty, intgrt_plain 
      double precision, external        :: intgrt_low 
      
      integer, parameter      :: nquad = 96 
      !integer :: i ; double precision :: lim(6)
                                                       
      ! This is needed by other routines               
      a_cmn = a 
      ! Check limits                                                    
      if ((hi_cut_implem .and. a > r_cut_hi) .or. a < r_cut_lo) then 
         write(0,*) "Bad value of parent size in function: lifetime" 
         write(0,*) "Parent:",a,"r_cut_hi:",r_cut_hi 
         write(0,*) "r_cut_lo:", r_cut_lo 
         stop 
      endif 
                                                                        
      ! Do the upper part of the integral: a to infinity or r_cut_hi
      ! Uses r -> a/t transform
      if (hi_cut_implem) then 
         t_lo = a/r_cut_hi 
      else 
         t_lo = 0.0d0 
      endif 
      t_hi = 1.0d0 
      lifetime = my_dgquad(intgrt_to_infty,t_lo,t_hi,nquad) 
                                                                        
      ! Do the middle part of the integral: a/2 to a 
      ! This is done separately because of cusp at a/2 (on positive side)
      ! No transform used
      t_lo = max(a/2.0d0, r_cut_lo) 
      lifetime = lifetime + my_dgquad(intgrt_plain,t_lo,a,nquad) 
                                                                        
                                                                        
      ! Do the bottom part if necessary                                 
      ! If the higher cutoff is relevant, then divide region into       
      ! two parts - one where it is relevant, one where it is not 
      ! Uses r -> a e^(-t) transform      
      if (r_cut_lo < a/2.0d0) then 
         t_lo = log(2.0d0) 
         if (hi_cut_implem .and. 1.5d0*a > r_cut_hi) then 
            t_hi = log(a/max(r_cut_lo, r_cut_hi-a)) 
            lifetime = lifetime + my_dgquad(intgrt_low,t_lo,t_hi,nquad) 
            t_lo = t_hi 
         endif 
         t_hi = log(a/r_cut_lo) 
         if ((t_hi - t_lo) > 5.0d0*epsilon(t_lo)*t_lo) then 
            lifetime = lifetime + my_dgquad(intgrt_low,t_lo,t_hi,nquad) 
         endif 
      endif 
                                                                        
                                    ! once for each half plane          
      lifetime = 2.0d0 * lifetime 
end function lifetime
                                                                        
                                                                        
                                                                        
                                                                        
!====================================================================== 
!     Return the emission rate transformed for infinite interval        
!====================================================================== 
function intgrt_to_infty(t) 
      use life_intgn 
      use cuts 
      double precision intgrt_to_infty, t 
      double precision r, p_true 
                                                                        
      r = a_cmn/t 
      intgrt_to_infty = p_true(a_cmn,r,r_cut_hi,hi_cut_implem)&
           & * a_cmn / t**2 
end function intgrt_to_infty
                                                                        
                                                                        
!====================================================================== 
!     Just return the emission rate as normally                         
!====================================================================== 
      function intgrt_plain(r) 
      use life_intgn 
      use cuts 
!      use life_intgn                                                   
      double precision intgrt_plain, r, p_true 
                                                                        
      intgrt_plain = p_true(a_cmn, r,r_cut_hi,hi_cut_implem) 
      end function intgrt_plain 
                                                                        
                                                                        
!====================================================================== 
!     Return emission rate transformed for lowest interval              
!====================================================================== 
      function intgrt_low(t) 
      use cuts 
      use life_intgn 
      double precision intgrt_low, t, r, p_true 
                                                                        
      r = a_cmn * exp (-t) 
      intgrt_low = r * p_true(a_cmn, r,r_cut_hi,hi_cut_implem) 
      end function intgrt_low                                      




!----------------------------------------------------------------------
! fortran 90 routine to perform an integration by gaussian quadrature of
! the function f, between the limits a and b, using n points.
! Currently only n = 96 is supported
! Based on similarly named (dgquad) routine in the CERNLIB library
! (Gives exactly same answer for n = 96)
! GPS 6/11/95 (CCN8 11)
!----------------------------------------------------------------------
double precision function my_dgquad(f, a, b, n)
      implicit none
      double precision, external   :: f
      double precision, intent(in) :: a, b
      integer,          intent(in) :: n
      !------------------------------------------------------------
      integer, parameter :: n_allwd = 96
      integer            :: i
      double precision   :: u(n_allwd / 2), v(n_allwd / 2)
      double precision   :: alfa, beta, delta, sm
      double precision, parameter :: hf = 0.5d0
      !----- n_allwd = 96 ---------
      DATA U( 1)/9.9968950388323077D-1/, V(1)/7.9679206555201243D-4/
      DATA U( 2)/9.9836437586318168D-1/, V(2)/1.8539607889469217D-3/
      DATA U( 3)/9.9598184298720929D-1/, V(3)/2.9107318179349464D-3/
      DATA U( 4)/9.9254390032376262D-1/, V(4)/3.9645543384446867D-3/
      DATA U( 5)/9.8805412632962380D-1/, V(5)/5.0142027429275177D-3/
      DATA U( 6)/9.8251726356301468D-1/, V(6)/6.0585455042359617D-3/
      DATA U( 7)/9.7593917458513647D-1/, V(7)/7.0964707911538653D-3/
      DATA U( 8)/9.6832682846326421D-1/, V(8)/8.1268769256987592D-3/
      DATA U( 9)/9.5968829144874254D-1/, V(9)/9.1486712307833866D-3/
      DATA U(10)/9.5003271778443764D-1/, V(10)/1.0160770535008416D-2/
      DATA U(11)/9.3937033975275522D-1/, V(11)/1.1162102099838499D-2/
      DATA U(12)/9.2771245672230869D-1/, V(12)/1.2151604671088320D-2/
      DATA U(13)/9.1507142312089807D-1/, V(13)/1.3128229566961573D-2/
      DATA U(14)/9.0146063531585234D-1/, V(14)/1.4090941772314861D-2/
      DATA U(15)/8.8689451740242042D-1/, V(15)/1.5038721026994938D-2/
      DATA U(16)/8.7138850590929650D-1/, V(16)/1.5970562902562291D-2/
      DATA U(17)/8.5495903343460146D-1/, V(17)/1.6885479864245172D-2/
      DATA U(18)/8.3762351122818712D-1/, V(18)/1.7782502316045261D-2/
      DATA U(19)/8.1940031073793168D-1/, V(19)/1.8660679627411467D-2/
      DATA U(20)/8.0030874413914082D-1/, V(20)/1.9519081140145022D-2/
      DATA U(21)/7.8036904386743322D-1/, V(21)/2.0356797154333325D-2/
      DATA U(22)/7.5960234117664750D-1/, V(22)/2.1172939892191299D-2/
      DATA U(23)/7.3803064374440013D-1/, V(23)/2.1966644438744349D-2/
      DATA U(24)/7.1567681234896763D-1/, V(24)/2.2737069658329374D-2/
      DATA U(25)/6.9256453664217156D-1/, V(25)/2.3483399085926220D-2/
      DATA U(26)/6.6871831004391615D-1/, V(26)/2.4204841792364691D-2/
      DATA U(27)/6.4416340378496712D-1/, V(27)/2.4900633222483610D-2/
      DATA U(28)/6.1892584012546857D-1/, V(28)/2.5570036005349361D-2/
      DATA U(29)/5.9303236477757208D-1/, V(29)/2.6212340735672414D-2/
      DATA U(30)/5.6651041856139717D-1/, V(30)/2.6826866725591762D-2/
      DATA U(31)/5.3938810832435744D-1/, V(31)/2.7412962726029243D-2/
      DATA U(32)/5.1169417715466767D-1/, V(32)/2.7970007616848334D-2/
      DATA U(33)/4.8345797392059636D-1/, V(33)/2.8497411065085386D-2/
      DATA U(34)/4.5470942216774301D-1/, V(34)/2.8994614150555237D-2/
      DATA U(35)/4.2547898840730055D-1/, V(35)/2.9461089958167906D-2/
      DATA U(36)/3.9579764982890860D-1/, V(36)/2.9896344136328386D-2/
      DATA U(37)/3.6569686147231364D-1/, V(37)/3.0299915420827594D-2/
      DATA U(38)/3.3520852289262542D-1/, V(38)/3.0671376123669149D-2/
      DATA U(39)/3.0436494435449635D-1/, V(39)/3.1010332586313837D-2/
      DATA U(40)/2.7319881259104914D-1/, V(40)/3.1316425596861356D-2/
      DATA U(41)/2.4174315616384001D-1/, V(41)/3.1589330770727167D-2/
      DATA U(42)/2.1003131046056720D-1/, V(42)/3.1828758894411006D-2/
      DATA U(43)/1.7809688236761860D-1/, V(43)/3.2034456231992663D-2/
      DATA U(44)/1.4597371465489694D-1/, V(44)/3.2206204794030251D-2/
      DATA U(45)/1.1369585011066592D-1/, V(45)/3.2343822568575928D-2/
      DATA U(46)/8.1297495464425559D-2/, V(46)/3.2447163714064269D-2/
      DATA U(47)/4.8812985136049731D-2/, V(47)/3.2516118713868836D-2/
      DATA U(48)/1.6276744849602970D-2/, V(48)/3.2550614492363166D-2/
      !------------------------------------------------------------

      if (n .ne. n_allwd) then
         write(0,*) "This value for n is not allowed in my_dgquad", n
         stop
      end IF
      alfa = hf * (b + a)
      beta = hf * (b - a)
      sm  = 0d0
      do i = 1, n / 2
         delta = beta * u(i)
         sm = sm + v(i) * (f(alfa + delta) + f(alfa - delta))
      end do
      my_dgquad = beta * sm
end function my_dgquad

