
!----------------------------------------------------------------------
! Given two sets of dipole pointers (ext_dips1 and ext_dips2) with n1 and
! n2 dipoles respectively, the routine returns the interaction between
! those two sets at all points on a grid --- the array for the grid must
! be provided, but the program works out the limits (in impact parameter)
! of that grid (and returns the lower limit and grid spacing for the
! 'horizontal' and 'vertical' directions (gh_lo, gh_int, gv_lo, gv_int) 
!
! Work out the interaction region for each pair of dipoles: this has a
! centre (where the centres of the two dipoles overlap), an "extent", which
! will be twice the sum of the radii. Easiest approach is to then treat the
! interaction region as rectangular. For each grid element covered by a
! given interaction region the "potential" is evaluated and added to that
! already at that grid point.
!
! Normalisation is left out of the interaction until the final stage, to
! save one multiplication per potential evaluation
!
! GPS 16/11/95 (CCN8 35) (+ GPS 17/5/95 (CCN5 73) + 24/5/95 (CCN5 ??)
! based on routine from spread.f90 (CCN3 155-7)
!----------------------------------------------------------------------
subroutine onm_onm_grid(ext_dips1, n1, ext_dips2, n2, grid, gh_lo, gh_int,&
     & gv_lo, gv_int) 
      use gluons
      use constants
      implicit none
      !----------------------------------------------------------------
      ! Arguments
      type(dpl_pntr), dimension(:), intent(in)  :: ext_dips1, ext_dips2
      integer,                      intent(in)  :: n1, n2
      double precision,             intent(out) :: grid(0:,0:)
      double precision,             intent(out) :: gh_lo, gh_int
      double precision,             intent(out) :: gv_lo, gv_int
      !-- Useful type -------------------------------------------------
      type intrctn
	 complex(kind(1d0))   psn
	 double precision     r, ampltd
      end type intrctn
      !------------------------------------------------------------
      ! Local variables
      complex(kind(1d0)), allocatable :: grid_coord(:,:)
      type(intrctn)      region
      integer            grid_size, grid_ubounds(2)
      integer            i1, i2, n12, i, j, k         ! counters
      double precision   gh_hi, gv_hi   ! grid limits not sent to user
      complex(kind(1d0)) dipole_1(1:2), dipole_2(1:2)
      integer            ih_rgn_lo, ih_rgn_hi, iv_rgn_lo, iv_rgn_hi
      !--------------------------------------------------------------------

      grid_ubounds = ubound(grid)
      if (grid_ubounds(1) /= grid_ubounds(2)) then
         write(0,*) 'Grid for onium-onium interaction should be square'
         write(0,*) 'STOP'
         stop
      else
         grid_size = grid_ubounds(1)
      end if
      
      allocate(grid_coord(0:grid_size, 0:grid_size))

      ! set up initial values of limits which will be immediately replaced
      gh_lo =  huge(gh_lo)/2d0
      gv_lo =  gh_lo
      gh_hi = -huge(gh_hi)/2d0
      gv_hi =  gh_hi

      ! Establish the details (position and size) of each interaction
      ! region, so as to determine the extent of the grid for it to include all
      ! interaction regions
      n12  = n1 * n2
      do i1 = 1, n1   
         do i2 = 1, n2
            i = i1 + n1*(i2-1)
	    ! vector to add to 2nd dipole such that its centre is at the
	    ! same point as the centre of the first dipole
            region%psn = (ext_dips1(i1)%p%hi_y_gluon%psn + &
                                   ext_dips1(i1)%p%lo_y_gluon%psn - &
                                   ext_dips2(i2)%p%hi_y_gluon%psn - &
                                   ext_dips2(i2)%p%lo_y_gluon%psn)/2.0d0

	    ! The extent of the interaction region
            region%r = (ext_dips1(i1)%p%size + ext_dips2(i2)%p%size) * 2d0

	    ! Current values for limits of grid
            gh_lo = min (gh_lo, real(region%psn) - region%r)
            gh_hi = max (gh_hi, real(region%psn) + region%r)
            gv_lo = min (gv_lo, aimag(region%psn) - region%r)
            gv_hi = max (gv_hi, aimag(region%psn) + region%r)
         end do
      end do

      
      grid   = 0d0
      ! work out grid intervals
      gh_int = (gh_hi - gh_lo) / (grid_size + 1)
      gv_int = (gv_hi - gv_lo) / (grid_size + 1)

      ! set up grid coords -- may save much time, but will reduce
      ! programming hassle later on. Grid coords are based at center of
      ! each grid element. First index is the horizontal one.
      do j = 0, grid_size
         do k = 0, grid_size
            grid_coord(j,k) = cmplx(gh_lo + (j + 0.5d0) * gh_int, &
                 gv_lo + (k + 0.5d0) * gv_int)
         end do
      end do

      ! Sample each interaction region at points on the grid within region%r
      ! of the centre of the interaction region. 
      do i1 = 1, n1
	 do i2 = 1, n2
	    i = i1 + n1*(i2-1)   

	    ! Work out position and size of the region once again
	    ! (since it wasn't stored)

	    ! vector to add to 2nd dipole such that its centre is at the
	    ! same point as the centre of the first dipole
            region%psn = (ext_dips1(i1)%p%hi_y_gluon%psn + &
                                   ext_dips1(i1)%p%lo_y_gluon%psn - &
                                   ext_dips2(i2)%p%hi_y_gluon%psn - &
                                   ext_dips2(i2)%p%lo_y_gluon%psn)/2.0d0

	    ! The extent of the interaction region
            region%r = (ext_dips1(i1)%p%size + ext_dips2(i2)%p%size) * 2d0


	    ! These are the limits of the area on the 
	    ! grid where this interaction region will contribute
	    ih_rgn_lo = floor(( real(region%psn) - region%r - gh_lo)/gh_int)
	    ih_rgn_hi = floor(( real(region%psn) + region%r - gh_lo)/gh_int)
	    iv_rgn_lo = floor((aimag(region%psn) - region%r - gv_lo)/gv_int)
	    iv_rgn_hi = floor((aimag(region%psn) + region%r - gv_lo)/gv_int)

	    ! for case of rightmost or upmost dipole
	    ih_rgn_hi = min(ih_rgn_hi, grid_size)
	    iv_rgn_hi = min(iv_rgn_hi, grid_size)
	    
	    ! set up dipole positions in easily useble form (may waste some
	    ! time: but neater to program -- come back to it later?)
	    dipole_1(1) = ext_dips1(i1)%p%hi_y_gluon%psn
	    dipole_1(2) = ext_dips1(i1)%p%lo_y_gluon%psn
	    dipole_2(1) = ext_dips2(i2)%p%hi_y_gluon%psn
	    dipole_2(2) = ext_dips2(i2)%p%lo_y_gluon%psn

	    ! Add contributions from this interaction region to all
	    ! relevant elements of the grid.
            do j = ih_rgn_lo, ih_rgn_hi
               do k = iv_rgn_lo, iv_rgn_hi
                  grid(j, k) = grid(j, k) + potential(dipole_1,&
                       & dipole_2 + grid_coord(j,k))
               end do
            end do
            
	 end do
      end do


      ! Put in magnitude of interaction into grid
      grid     = grid * alpha_no_nc**2 / 2d0


contains



!***************************************************************************
! Routine to work out the potential between two dipoles with sources
! positioned at positions dipole_1 and dipole_2
!
! GPS 16/5/95 (CCN5 65)
!***************************************************************************
function potential(dipole_1, dipole_2)
     implicit none
     double precision      potential
     complex(kind(1d0)), intent(in)  :: dipole_1(1:2), dipole_2(1:2)
     double precision      numerator, denominator
     double precision,     parameter   :: max_potn = 1d3
     
     ! Work out square of absolute values of each quantity -- hopefully
     ! this is the fastest way of dealing with things. Requires factor of 4
     ! after the log though. 
     numerator    = abs_sqrd(dipole_2(2) - dipole_1(2)) * &
                    abs_sqrd(dipole_2(1) - dipole_1(1)) 
     denominator  = abs_sqrd(dipole_2(2) - dipole_1(1)) * &
                    abs_sqrd(dipole_2(1) - dipole_1(2))
     if (numerator == 0d0 .or. denominator == 0d0) then
        potential = max_potn
     else
        ! remember factor of 4 to take into account no sqrt in ratios
        potential = (log(numerator / denominator))**2 / 4d0 
     end if
     potential = min(potential, max_potn)
end function potential


! just a useful short cut to avoid some square roots
! and various extra multiplications
function abs_sqrd(vector)
      double precision abs_sqrd
      complex(kind(1d0)), intent(in) :: vector
               
      abs_sqrd = real(vector)**2 + aimag(vector)**2
end function abs_sqrd
          
end subroutine onm_onm_grid

