
!**********************************************************************
! Routine to initialise the parameters for the MC run. If a third command
! line argument is present then the universal parameters in 
! start_dir//arg3 will be used. The parameters specific to this run are
! the defaults specified in the parent program. 
!
! If no third command line argument is present then it is assumed that
! this is a continuation run and the all parameters are read in from th
! appropriate .prm file (the same is done if 3rd arg is a dash '-')
!
! To be called when data file uses FORMATTED i/o
! Backups are left to driver routine
!
! If n_new_events is present then it contains the value of arg2 (the
! number of new events to be processed in this run)
!
! If dat_append is present, and true, and this run is a continuation of a
! previous one, the data file is positioned so that any subsequent writes
! will be appended to it (unless of course, a rewind is done -- see
! write_params_fdat)
!
! GPS 25/11/94 (CCN3 83) + 23/5/95 (CCN5 97) + 4/11/95 (CCN8 7)
!**********************************************************************
subroutine read_params_fdat(onm_size, maxy, n_prev_events, n_new_events,&
     & dat_append)
      use sub_defs, except => read_params_fdat
      use cuts
      use iomodule
      use data_defs
      implicit none
      !------ The various parameters ---------------------
      double precision,  intent(out) :: onm_size, maxy
      integer,           intent(out) :: n_prev_events
      integer, optional, intent(out) :: n_new_events
      logical, optional, intent(in)  :: dat_append ! defaults to .false.
      !------ Needed for getting command line arguments --
      character*79                  :: start_file, prog_name
      integer                       :: i, j, lcl_iargc
      logical                       :: start_seqnc
      !------ In case seed needs to be set ---------------
      integer                       :: jseed(2)
      !---------------------------------------------------

      call openfiles_fdat

      ! Check the command line for a sufficient number of arguments
      i = lcl_iargc()
      if (i < 2) then 
	 call lcl_getarg(0,prog_name)
         write(0,*) ' Usage: ',prog_name(1:index(prog_name,' ')),&
	      & 'file_basename number_of_events [start_file | - ]'
         stop
      end if
      
      ! fix, so that if 3rd arg is absent, or just '-' then this run is a
      ! continuation 
      start_seqnc = .false.
      if (i > 2) then
         call lcl_getarg(3,start_file)
	 j = index(start_file,' ') - 1
         if ( start_file(1:j) /= '-' ) start_seqnc = .true.
      end if
      
      ! Decide whether it is coninuation or startup
      if (.not. start_seqnc) then
	 ! continuation
	 write(0,*) 'Continuing a previous evolution....'
	 read(iprm,*) jseed              ! Random Number Seed
	 read(iprm,*) r_cut_lo, r_cut_hi ! Cuts
	 read(iprm,*) onm_size, maxy    ! 
	 read(iprm,*) data_io
	 read(iprm,*) n_prev_events
      else
	 ! startup
	 write(0,*) 'Starting a new evolution'
	 open(unit = istartup, file=start_dir//start_file(1:j), status = 'old')

	 read(istartup, *) jseed
	 read(istartup, *) r_cut_lo, r_cut_hi
	 read(istartup, *) onm_size, maxy
	 n_prev_events = 0
      end if

      !-- determine whether it is necessary to rewind data device
      !-- if dat_append is true, it is still necessary to rewind data file
      !-- for a new run (i.e. get rid of old data file).
      if (.not. present(dat_append)) then
         rewind idat
      elseif ((.not. dat_append) .or. n_prev_events == 0) then
         rewind idat
      end if
         

      ! Make sure hwrgen not initialised with a 0 seed (use this as signal
      ! to use default seed).
      if (jseed(1) /= 0 .and. jseed(2) /= 0) then 
	 call hwrset(jseed)
	 write(0,*) 'HWRGEN has been initialised'
      else
	 write(0,*) 'HWRGEN will use its default seed'
      end if
      
      ! Get the number of events needed in this run
      if (present(n_new_events)) n_new_events = int_val_arg(2)
      
      ! initialise the tabulated dipole lifetimes as a function of size
      call init_lives
      write(0,*) 'Lifetimes have been initialised'

      ! initialise the dipole set (just nullify relevant pointers)
      call nullify_child_set

end subroutine read_params_fdat


!**********************************************************************
!     Routine to be called at end of evolution to write the updated
!     parameters to a file: main routine must write the data to dat_dev
!     Include details of which device to read data from
!
! If param_arg and dat_arg are not specified then the default devices are
! deemed to be the ones used for reading and writing pars and data. By
! default dat_arg is rewound, but if rwnd_dat is present and .false. then
! this is not done (i.e. allowing data to be appended to the file)
!
! GPS 25/11/94 (CCN3 83) + 4/11/95 (CCN7 7)
!**********************************************************************
subroutine write_params_fdat(onm_size, maxy, n_events, &
     &                               param_arg, dat_arg, rwnd_dat)
      use sub_defs, except => write_params_fdat
      use cuts
      use data_defs
      implicit none
      !-- arguments ---------------------------------------
      integer,           intent(in)  :: n_events
      double precision,  intent(in)  :: onm_size, maxy
      integer, optional, intent(in)  :: param_arg, dat_arg
      logical, optional, intent(in)  :: rwnd_dat
      !-------- for record keeping ------------------------
      character*79           :: prog_name
      integer                :: time(8), dummy, i, j
      integer                :: param_dev, dat_dev
      !------- Local vars ---------------------------------
      integer, dimension(2) :: jseed

      ! This is a workaround s othat the routine doesn't need access to
      ! the iomodule (which could cause problems when one of the args is
      ! from there)
      call get_io_devs(i,j,dummy)
      if (present(param_arg)) then
         param_dev = param_arg
      else
         param_dev = i
      end if
      if (present(dat_arg)) then
         dat_dev = dat_arg
      else
         dat_dev = j
      end if

      rewind param_dev
      !-- There are situations where one does not want to rewind the data
      !-- device
      if (present(rwnd_dat)) then
         if (rwnd_dat) rewind dat_dev
      else
         rewind dat_dev
      end if
      
      call hwrget(jseed)
      write(param_dev,*) jseed       ! Random Number Seed
      write(param_dev,*) r_cut_lo, r_cut_hi
      write(param_dev,*) onm_size, maxy
      write(param_dev,*) data_io
      write(param_dev,*) n_events

      ! write program name so that param file contains a record of which
      ! program created it (i.e. so that you can go back and try to
      ! interpret the parameter names)
      call lcl_getarg(0,prog_name)
      write(param_dev,*) 'This file was last changed by ',&
	   & prog_name(1:index(prog_name,' ')),' on'

      ! Write out a time and date stamp
      call date_and_time(values=time) ! get the full date and time
      write(param_dev,15) time(3),time(2),time(1),time(5),time(6)&
	   & ,time(7)
15    format(i2,'/',i2.2,'/',i4.4,'   ',i2.2,':',i2.2,':',i2.2)
      end subroutine write_params_fdat
