!**********************************************************************
! Routine to initialise the parameters for the MC run. If a third command
! line argument is present then the universal parameters in 
! start_dir//arg3 will be used. The parameters specific to this run are
! the defaults specified in the parent program. 
!
! If no third command line argument is present then it is assumed that
! this is a continuation run and the all parameters are read in from th
! appropriate .prm file
!
! GPS 25/11/94 (CCN3 83) + 23/5/95 (CCN5 97)
!**********************************************************************
subroutine read_params_ufdt(prnt_size, maxy, n_prev_events, n_new_events)
      use sub_defs, except => read_params_ufdt      
      use cuts
      use iomodule
      use data_defs
      use binning
      implicit none
      !------ The various parameters ---------------------
      double precision,  intent(out) :: prnt_size, maxy
      integer,           intent(out) :: n_prev_events
      integer, optional, intent(out) :: n_new_events
      !------ Needed for getting command line arguments --
      character*79                  :: start_file, prog_name
      integer                       :: i, j, lcl_iargc
      logical                       :: start_seqnc
      !------ In case seed needs to be set ---------------
      integer                       :: jseed(2)
      !---------------------------------------------------

      call openfiles_ufdt

      ! Check the command line for a sufficient number of arguments
      i = lcl_iargc()
      if (i < 2) then 
	 call lcl_getarg(0,prog_name)
         write(0,*) ' Usage: ',prog_name(1:index(prog_name,' ')),&
	      & 'file_basename number_of_events [start_file | - ]'
         stop
      end if
      
      ! fix, so that if 3rd arg is absent, or just '-' then this run is a
      ! continuation 
      start_seqnc = .false.
      if (i > 2) then
         call lcl_getarg(3,start_file)
	 j = index(start_file,' ') - 1
         if ( start_file(1:j) /= '-' ) start_seqnc = .true.
      end if

      ! Decide whether it is coninuation or startup
      if (.not. start_seqnc) then
	 ! continuation
	 write(0,*) 'Continuing a previous evolution....'
	 read(iprm,*) jseed              ! Random Number Seed
	 read(iprm,*) r_cut_lo, r_cut_hi ! Cuts
	 read(iprm,*) prnt_size, maxy    ! 
	 read(iprm,*) data_io
	 read(iprm,*) n_prev_events
         read(iprm,*) bn
      else
	 ! startup
	 write(0,*) 'Starting a new evolution'
	 open(unit = istartup, file=start_dir//start_file(1:j), status = 'old')

	 read(istartup, *) jseed
	 read(istartup, *) r_cut_lo, r_cut_hi
	 read(istartup, *) prnt_size, maxy
	 n_prev_events = 0
         !---- set up these vars for bn% -----
         bn%nf = bn%n_f_lo + bn%n_f_hi
         bn%nr = bn%n_r_lo + bn%n_r_hi
      end if


      ! Make sure hwrgen not initialised with a 0 seed (use this as signal
      ! to use default seed).
      if (jseed(1) /= 0 .and. jseed(2) /= 0) then 
	 call hwrset(jseed)
	 write(0,*) 'HWRGEN has been initialised'
      else
	 write(0,*) 'HWRGEN will use its default seed'
      end if
      
      
      ! Get the number of events needed in this run
      if (present(n_new_events)) n_new_events = int_val_arg(2)

      !------------------------------------------------------------
      ! initialise the tabulated dipole lifetimes as a function of size
      call init_lives
      write(0,*) 'Lifetimes have been initialised'

      ! nullify the pointers to children for the set of parent dipoles
      call nullify_child_set

end subroutine read_params_ufdt



!**********************************************************************
!     Routine to be called at end of evolution to write the updated
!     parameters to a file: main routine must write the data to dat_dev
!     Include details of which device to read data from
!
! GPS 25/11/94 (CCN3 83)
!**********************************************************************
subroutine write_params_ufdt(prnt_size, maxy, n_events, param_arg,&
           & ufdt_arg)
      use sub_defs, except => write_params_ufdt
      use cuts
      use data_defs
      use binning
      implicit none
      integer,           intent(in)  :: n_events
      double precision,  intent(in)  :: prnt_size, maxy
      integer, optional, intent(in)  :: param_arg, ufdt_arg
      !-------- for record keeping ----------------------
      character*79                 :: prog_name
      integer                      :: time(8), i, j, dummy
      integer                      :: param_dev, ufdt_dev
      !------- Local vars ----------------------------------
      integer, dimension(2) :: jseed

      ! This is a workaround s othat the routine doesn't need access to
      ! the iomodule (which could cause problems when one of the args is
      ! from there)
      call get_io_devs(i, dummy, j)
      if (present(param_arg)) then
         param_dev = param_arg
      else
         param_dev = i
      end if
      if (present(ufdt_arg)) then
         ufdt_dev = ufdt_arg
      else
         ufdt_dev = j
      end if

      rewind param_dev
      rewind ufdt_dev
      
      call hwrget(jseed)
      write(param_dev,*) jseed       ! Random Number Seed
      write(param_dev,*) r_cut_lo, r_cut_hi
      write(param_dev,*) prnt_size, maxy
      write(param_dev,*) data_io
      write(param_dev,*) n_events
      write(param_dev,*) bn


      ! write program name so that param file contains a record of which
      ! program created it (i.e. so that you can go back and try to
      ! interpret the parameter names)
      call lcl_getarg(0,prog_name)
      write(param_dev,*) 'This file was last changed by ',&
	   & prog_name(1:index(prog_name,' ')),' on'

      ! Write out a time and date stamp
      call date_and_time(values=time) ! get the full date and time
      write(param_dev,15) time(3),time(2),time(1),time(5),time(6)&
	   & ,time(7)
15    format(i2,'/',i2.2,'/',i4.4,'   ',i2.2,':',i2.2,':',i2.2)
end subroutine write_params_ufdt




!--- OBSOLETE ROUTINES BUT STILL NEEDED FOR BACKWARDS COMPATABILITY

!------------------------------------------------------------------------
! Emulates function of an old routine to read in old unformatted data for
! analysis -- by calling read_params_old_ufdt and processing the args
!
! GPS 9/11/95
!------------------------------------------------------------------------
subroutine rp_ufdt_obs(prnt_size, maxy, n_events, start_seqnc)
      use sub_defs
      implicit none
      !------ The various parameters ---------------------
      integer,          intent(out) :: n_events
      double precision, intent(out) :: prnt_size, maxy
      logical,          intent(out) :: start_seqnc
      !------------------------------------------------------------
      integer n_new_events

      call read_params_ufdt(prnt_size, maxy, n_events, n_new_events)

      start_seqnc = (n_events == 0)
end subroutine rp_ufdt_obs







