
!----------------------------------------------------------------------
! Specific to these version of these routines
!
! New gluons and dipoles are dynamically allocated
!
! The limit on dipole numbers is 'artificial' (ie not related to sizes of
! any arrays) -- the number of dipoles should be less than max_gluons*2.
! If calling from evolve_onium, and it reaches this limit then this tree
! of dipoles is deallocated and then reevolved
!----------------------------------------------------------------------

!-------------------------------------------------------------------------
! Routine to start off the evolution of an onium. Evolution is carried out
! by evolve_alloc, but it is necessary to set the dipole index and to
! check for the number of evolved dipoles being too large
!
! GPS 7/11/95 (CCN8 15)
!----------------------------------------------------------------------
subroutine evolve_onium(onium, maxy, dpl_index)
      use gluons
      use emergency
      implicit none
      type(dipole),     intent(inout) :: onium
      double precision, intent(in)    :: maxy
      integer,          intent(out)   :: dpl_index
      !------------------------------------------------------------

      do
         dpl_index = 1
         call evolve_alloc(onium, maxy, dpl_index)
         if (dpl_index < max_gluons * 2) exit
         !-- following should only be called rarely
         write(0,*) 'Too many dipoles (>',dpl_index,') in this event'
         write(0,*) 'Limit is ',max_gluons*2
         write(0,*) 'The configuration will be generated again'
         write(0,*) 'Number of evolved dipoles is', n_dip_emergency
         call dealloc_tree(onium)
      end do
end subroutine evolve_onium


!=========================================================================
! Routine to evolve an initial dipole up to a rapidity maxy, allocating
! dipole and gluon structures as they are needed -- works out one dipole
! at a time and then calls itself to evolve the child ones. Index of
! parent is set to dipole_n. See p123 for details of numbering.
!
! Need to tidy up the determination of the dipole sizes ?          
! (it is being worked out too often?)
!                                                                       
! GPS 16/11/94 (CCN3 c. 30-40?)  + 3/10/95 (CCN7 123)
!====================================================================== 
recursive subroutine evolve_alloc(parent, maxy, dpl_index) 
      use gluons 
      use cuts 
      implicit none 
      type(dipole), intent(inout)  ::  parent 
      double precision, intent(in) ::  maxy 
      integer, intent(inout)       ::  dpl_index
! Internal Variables                                                    
      type(gluon), pointer         ::  current_gluon 
      !type(dipole), pointer        ::  dipole_1, dipole_2 
      double precision b, phi, weight, newrap, get_life
      double precision x, hwrgen
      integer i
      
      !-- set up index for this dipole -----
      parent%index = dpl_index
                                                                        
      x = hwrgen(i)
      newrap = parent%hi_y_gluon%rapidity - log(x)*get_life(parent%size)
      !-- also trap case where number of dipoles has exceeded limit
      if (newrap .ge. maxy .or. dpl_index >= max_gluons * 2) then 
         ! no more branching    
         nullify(parent%child_1, parent%child_2) 
         ! Potentially call histogramming routines at this point
         ! (but not for case of exceeding limit on dipole number)
         return 
      else 
         !-- new version -- allocate things together -- might be faster?
         allocate (current_gluon, parent%child_1, parent%child_2)
         current_gluon%rapidity = newrap 
      endif 
                                                                        
 10   call gnrt_pnt(parent%size,b,phi,weight) 
      call get_glu_psn(parent%lo_y_gluon%psn,                           &
     &                 parent%hi_y_gluon%psn,                           &
     &                  parent%size, current_gluon%psn,                 &
     &                  b, phi)                                         
                                                                        
                                            
! Set up the parameters of the first child dipole                       
      parent%child_1%lo_y_gluon => parent%lo_y_gluon 
      parent%child_1%hi_y_gluon => current_gluon 
      parent%child_1%size = abs(parent%lo_y_gluon%psn                   &
     &                               - current_gluon%psn)               
                                                                        
! Set up the parameters of the second child dipole                      
      parent%child_2%lo_y_gluon => parent%hi_y_gluon 
      parent%child_2%hi_y_gluon => current_gluon 
      parent%child_2%size = abs(parent%hi_y_gluon%psn                   &
     &                               - current_gluon%psn)               
                                                                        
      ! It looks as if numerical error is sometimes producing a dipole
      ! which is smaller than the lower cutoff. This very ugly kludge
      ! rejects that newly generated gluon and goes back to line 10
      ! (Ugh...) to cause a new one to be generated. It also produces an
      ! error message (the routine emergency stop  doesn't actually cause
      ! the program to stop anymore)
      if (parent%child_1%size < r_cut_lo .or. &
     &    parent%child_2%size < r_cut_lo) then
         print *, "Error: dipole size is too small"
         print *, "r_cut_lo:",r_cut_lo, ", b:",b
         print *, "phi:",phi
         call printall(parent%child_1, 1)
         call printall(parent%child_2, 2)
         call emergency_stop
         goto 10
      endif

      !--- Recurse ----------
      dpl_index = dpl_index + 1
      call evolve_alloc(parent%child_1,maxy,dpl_index) 
      dpl_index = dpl_index + 1
      call evolve_alloc(parent%child_2,maxy,dpl_index) 

!!!!!! DEBUG
      contains 
      subroutine printall(dip, i)
      type(dipole), intent(in) :: dip
      integer, intent(in)      :: i
 
      print *,'---------------------------------------------------------------'
      print *,'Child',i,' Parent dpl_index',dpl_index
      print *,'Size:', dip%size
      print *,'Hi y Gluon:', dip%hi_y_gluon%psn, dip%hi_y_gluon%rapidity
      print *,'Lo y Gluon:', dip%lo_y_gluon%psn, dip%lo_y_gluon%rapidity
      print *,'---------------------------------------------------------------'
      print *, ' '
      end subroutine printall
      
      END                                           
                                                                        
                                                                        
!====================================================================== 
!    Function to return the rapidity step                               
!    Very basic at the moment, with no alpha_s, etc... or upper cutoffs 
!    Lower cutoff is approximated as small                              
!                                                                       
! GPS 10/11/94 (CCN3 29)                                                
!====================================================================== 
!      function rpdty_step(size) 
!      use cuts 
!      implicit none 
!      double precision rpdty_step 
!      double precision, intent(in)    :: size 
!      double precision hwrgen, h
!      integer i 
! 
!      h = hwrgen(i)
!      rpdty_step = - log(h)/log(size/r_cut_lo) 
!      END                                           

