!======================================================================
! Clear the whole dipole set -- should be called by the initialisation
! routines 
!
! GPS 2/10/95 (CCN7 121)
!======================================================================
subroutine nullify_child_set
      use gluons
      implicit none
      integer i

      !-- gluons shouldn't need to be nullified?
      do i = 1, max_set
         nullify(dipole_set(i)%child_1, dipole_set(i)%child_2)
         nullify(dipole_set(i)%hi_y_gluon,dipole_set(i)%lo_y_gluon)
      end do
end subroutine nullify_child_set

!********************************************************************** 
! Return a pointer to a dipole centred on (0,0), of size onm_sz and with
! an orientation phi.
! Any children and their gluons are deallocated
! The dipole is actually taken from an array of dipoles provided in the
! gluon module, while the gluons are dynamically allocated.
!                                                                       
! GPS 16/11/94 (CCN3 47)  --> 7/11/95                                      
!********************************************************************** 
subroutine init_onium(onium, onm_sz, phi) 
      use gluons 
      use evoln_intrfc
      use cuts
      implicit none 
      type(dipole), pointer         :: onium
      double precision,  intent(in) :: onm_sz, phi
      !-------------------------------------------------------------------

      if (onm_sz < r_cut_lo) then
         write(0,*) "Onia may not be initialised with sizes less than the "
         write(0,*) "lower cutoff (",r_cut_lo,")"
         write(0,*) "STOP"
         stop
      end if
      

      dipole_n = dipole_n + 1 
      onium => dipole_set(dipole_n)

      !-- Remove any tree associated with this dipole
      call dealloc_tree(onium)
      allocate(onium%lo_y_gluon)
      allocate(onium%hi_y_gluon)
                                                                        
      onium%lo_y_gluon%psn = cmplx(cos(phi),sin(phi))*onm_sz/2d0 
      onium%lo_y_gluon%rapidity = 0.0d0 
      onium%hi_y_gluon%psn = -cmplx(cos(phi),sin(phi))*onm_sz/2d0 
      onium%hi_y_gluon%rapidity = 0.0d0 
      gluon_n = gluon_n + 2 
                                                                        
      onium%size = abs(onium%lo_y_gluon%psn - onium%hi_y_gluon%psn)
end subroutine init_onium



!======================================================================
! Routine to deallocate all the resources held in a tree of dipoles, in
! such a way that there is never any attempt to free the same area twice.
! The bottom dipole is left in place (but not its gluons), in case it is
! not deallocatable
!
! GPS 15/10/95 (CCN7 143)
!======================================================================
subroutine dealloc_tree(btm_dpl)
      use gluons
      implicit none
      !-----------------------------------------------------------------
      type(dipole), intent(inout)  :: btm_dpl
      !-----------------------------------------------------------------

      !-- deal with lowest gluons -- they won't be allocated on the first
      !-- call 
      if (associated(btm_dpl%lo_y_gluon)) then 
         deallocate (btm_dpl%lo_y_gluon)
      end if
      if (associated(btm_dpl%hi_y_gluon)) then 
         deallocate (btm_dpl%hi_y_gluon)
      end if

      !-- this routine will deallocate a hi y gluon
      if (associated(btm_dpl%child_1)) then 
         call dlc_tree_gln(btm_dpl%child_1)
         deallocate (btm_dpl%child_1)
      end if
      !-- this routine will deallocate no gluons
      if (associated(btm_dpl%child_2)) then 
         call dlc_tree(btm_dpl%child_2)
         deallocate (btm_dpl%child_2)
      end if
   
contains

!----------------------------------------------------------------------
! The gluon pointed to by hi_y_gluon will be encountered here for the
! first time -- deallocate it as well as any child dipoles they have been
! treated with dealloc_tree
!----------------------------------------------------------------------
recursive subroutine dlc_tree_gln(btm_dpl)
      use gluons 
      implicit none
      !-----------------------------------------------------------------
      type(dipole), intent(inout)  :: btm_dpl
      !-----------------------------------------------------------------

      !-- deallocate only the hi y gluon (whose first occurrence in the
      !-- deallocation series will be at this point)
      deallocate (btm_dpl%hi_y_gluon)

      !-- this routine will deallocate a hi y gluon
      if (associated(btm_dpl%child_1)) then 
         call dlc_tree_gln(btm_dpl%child_1)
         deallocate (btm_dpl%child_1)
      end if
      !-- this routine will deallocate no gluons
      if (associated(btm_dpl%child_2)) then 
         call dlc_tree(btm_dpl%child_2)
         deallocate (btm_dpl%child_2)
      end if
end subroutine dlc_tree_gln

!----------------------------------------------------------------------
! Both gluons in this dipole have already been deallocated -- just
! deallocate the child dipoles
!----------------------------------------------------------------------
recursive subroutine dlc_tree(btm_dpl)
      use gluons 
      implicit none
      !-----------------------------------------------------------------
      type(dipole), intent(inout)  :: btm_dpl
      !-----------------------------------------------------------------

      !-- this routine will deallocate a hi y gluon
      if (associated(btm_dpl%child_1)) then 
         call dlc_tree_gln(btm_dpl%child_1)
         deallocate (btm_dpl%child_1)
      end if
      !-- this routine will deallocate no gluons
      if (associated(btm_dpl%child_2)) then 
         call dlc_tree(btm_dpl%child_2)
         deallocate (btm_dpl%child_2)
      end if
end subroutine dlc_tree

end subroutine dealloc_tree








      
      
      

