!----------------------------------------------------------------------
! Specific to these versions of these routines:
!
! New Gluons and dipoles come from a pre allocated array. The global
! counters gluon_n and dipole_n keep track of the furthest part of the
! array containing gluons or dipoles.
!
! The limit on dipole numbers comes from the size of the array -- if the
! gluon_n pointer ever becomes larger than max_gluons then
! further evolution stops. If the evolution was started through
! evolve_onium then the evolution is restarted again.
!----------------------------------------------------------------------



!-------------------------------------------------------------------------
! Routine to start off the evolution of an onium. Evolution is carried out
! by evolve_fixed, but it is necessary to set the dipole index and to
! check for the number of evolved dipoles being too large
!
! GPS 7/11/95 (CCN8 15)
!----------------------------------------------------------------------
subroutine evolve_onium(onium, maxy, dpl_index)
      use gluons
      use evoln_intrfc
      use emergency
      implicit none
      type(dipole),     intent(inout) :: onium
      double precision, intent(in)    :: maxy
      integer,          intent(out)   :: dpl_index
      !------------------------------------------------------------
      integer   dipole_n_strt, gluon_n_strt

      dipole_n_strt = dipole_n
      gluon_n_strt  = gluon_n
      do
         dpl_index = 1
         call evolve_fixed(onium, maxy, dpl_index)
         if (gluon_n < max_gluons) exit
         !-- following should only be called rarely -- if arrays are big
         !-- enough. 
         write(0,*) 'Too many gluons (>=',gluon_n,') in this event'
         write(0,*) 'Limit is ',max_gluons
         write(0,*) 'The configuration will be generated again'
         write(0,*) 'Number of evolved dipoles is', n_dip_emergency
         dipole_n = dipole_n_strt
         gluon_n  = gluon_n_strt
      end do
end subroutine evolve_onium
      

!====================================================================== 
! Recursive subroutine to evolve the parent dipole, by emitting a          
! gluon (as long as its rapidity < maxy) and then evolve the 
! two resulting dipoles.                                            
!
! Dipoles and gluons come from fixed size pre allocated arrays. The
! current position in each array is denoted dipole_n and gluon_n.
!
! Dipoles are also allocated an index number, which should be unique for
! that onium (as long as dpl_index is always supplied for any subsequent
! evolution).
!
! Need to implement a check on overflow of matrices.....
!                                                                       
! GPS 16/11/94 (CCN3 c. 30-40?)
!====================================================================== 
recursive subroutine evolve_fixed(parent,maxy,dpl_index) 
      use gluons 
      use evoln_intrfc
      use cuts 
      implicit none 
      type(dipole),     intent(inout) ::  parent 
      double precision, intent(in)    ::  maxy 
      integer,          intent(inout) ::  dpl_index
      !-- Internal Variables ------------------------------
      type(gluon), pointer        ::  current_gluon 
      !type(dipole), pointer       ::  dipole_1, dipole_2 
      double precision b, phi, weight, newrap, get_life
      double precision x, hwrgen
      integer i
      !----------------------------------------------------

      !-- set up index for this dipole
      parent%index = dpl_index

      x = hwrgen(i)
      newrap = parent%hi_y_gluon%rapidity - &
     &    log(x)*get_life(parent%size)
                                                                        
      !-- test for reaching limit on rapidity, and limit on max dipole
      ! numbers
      if (newrap .ge. maxy .or. gluon_n >= max_gluons) then 
         !-- no more branching  
         nullify(parent%child_1, parent%child_2) 
         return 
      else
         gluon_n = gluon_n + 1 
         current_gluon => gluon_set(gluon_n) 
         current_gluon%rapidity = newrap 
      endif 
                                                                        
 10   call gnrt_pnt(parent%size,b,phi,weight) 
      call get_glu_psn(parent%lo_y_gluon%psn,                           &
     &                 parent%hi_y_gluon%psn,                           &
     &                  parent%size, current_gluon%psn,                 &
     &                  b, phi)                                         
                                                                        
!      write(6,*) real(parent%hi_y_gluon%psn),                          
!     &              aimag(parent%hi_y_gluon%psn),                      
!     &               parent%hi_y_gluon%rapidity,                       
!     &               gluon_n                                           
                                                                        
                                                                        
! Set up the parameters of the first child dipole  
      dipole_n = dipole_n + 1 
      parent%child_1 => dipole_set(dipole_n) 
      parent%child_1%lo_y_gluon => parent%lo_y_gluon 
      parent%child_1%hi_y_gluon => gluon_set(gluon_n) 
      parent%child_1%size = abs(parent%lo_y_gluon%psn                   &
     &                               - current_gluon%psn)               
                                                                        
! Set up the parameters of the second child dipole                      
      dipole_n = dipole_n + 1 
      parent%child_2 => dipole_set(dipole_n) 
      parent%child_2%lo_y_gluon => parent%hi_y_gluon 
      parent%child_2%hi_y_gluon => gluon_set(gluon_n) 
      parent%child_2%size = abs(parent%hi_y_gluon%psn                   &
     &                               - current_gluon%psn)               
                                                                        
      ! It looks as if numerical error is sometimes producing a dipole
      ! which is smaller than the lower cutoff. This very ugly kludge
      ! rejects that newly generated gluon and goes back to line 10
      ! (Ugh...) to cause a new one to be generated. It also produces an
      ! error message (the routine emergency stop  doesn't actually cause
      ! the program to stop anymore)
      if (parent%child_1%size < r_cut_lo .or. &
     &    parent%child_2%size < r_cut_lo) then
         print *, "Error: dipole size is too small"
         print *, "r_cut_lo:",r_cut_lo, ", b:",b
         print *, "phi:",phi
         call printall(parent%child_1, 1)
         call printall(parent%child_2, 2)
         call emergency_stop
         dipole_n = dipole_n - 2     ! Very ugly kludge
         goto 10
      endif

! Recurse 
      dpl_index = dpl_index + 1
      call evolve_fixed(parent%child_1,maxy,dpl_index) 
      dpl_index = dpl_index + 1
      call evolve_fixed(parent%child_2,maxy,dpl_index) 

!!!!!! DEBUG
      contains 
      subroutine printall(dip, i)
      type(dipole), intent(in) :: dip
      integer, intent(in)      :: i
 
      print *,'---------------------------------------------------------------'
      print *,'Child',i,' Dipole_n',dipole_n,' gluon_n', gluon_n
      print *,'Size:', dip%size
      print *,'Hi y Gluon:', dip%hi_y_gluon%psn, dip%hi_y_gluon%rapidity
      print *,'Lo y Gluon:', dip%lo_y_gluon%psn, dip%lo_y_gluon%rapidity
      print *,'---------------------------------------------------------------'
      print *, ' '
      end subroutine printall
      
end subroutine evolve_fixed




