!**********************************************************************
!     Module that defines a data type to contain all info relating to
!     data that needs to be i/o-d
! GPS 11/12/94 (CCN3 113)
!**********************************************************************
module data_defs
      !-- see README for details of type defn
      type data_prms
	 real(kind(1d0))  :: y_int
	 integer           :: n_y, n_pom 
      end type data_prms

      data data_io /data_prms(0.5d0, 5, 4)/ 
      type(data_prms)        data_io

end module data_defs
   

!*************************************************************************
!     Routine to look at interactions between two independently 
!     generated sets of dipoles -> including unitarity corrections.
!
!     ampltd(0:n,i_y) contains cross section for 1 to n pomeron exchange
!     (ampltd(1:n,i_y)), the unitarised x-sectn (ampltd(0,i_y)).
!     i_y is the index indicating which value of y is relevant.
!
! GPS 19/5/95 (CCN5 81) + 4/11/95 (CCN8 11)
!****************************************************************************
program onm_onm_ftot
      use emergency             !DEBUG
      use gluons                ! Definitions for gluons and dipoles 
      use sub_defs              ! Some interface blocks 
      use iomodule              ! Open files and get command line args
      use data_defs             ! Def for data type containing data i/o info
      implicit none 
      !--- vars which are generally needed ---------------------------------
      real(kind(1d0))       onm_size, maxy, y
      integer                n_prev_events, n_new_events
      !--- vars specific to this analysis ----------------------------------
      type(dpl_pntr), allocatable      :: ext_dips1(:), ext_dips2(:)
      type(dipole),   pointer          :: onium1, onium2
      real(kind(1d0)), allocatable    :: ampltd(:,:)
      real(kind(1d0)), allocatable    :: ampltd_evnt(:)
      integer                i, ii, i_y
      integer                n1, n2
      !----------------------------------------------------------------------

      call read_all

      write(0,*) 'Am about to evolve'

      do ii = 1,n_new_events
         call reset_counters
         !-- first onium
         call init_onium_rnd_phi(onium1, onm_size)
         call evolve_onium(onium1, maxy, n1)
         allocate(ext_dips1(n1/2 + 1))
         !-- second onium
         call init_onium(onium2, onm_size, 0d0)
         call evolve_onium(onium2, maxy, n2)
         allocate(ext_dips2(n2/2 + 1))
	 
	 ! For each value of y, extract the two sets of dipoles and then
	 ! work out the cross sections
         do i_y = 1, data_io%n_y
	    y  = maxy + (i_y - data_io%n_y) * data_io%y_int

            call extract_onium_dpls(onium1, y, ext_dips1, n1)
            call extract_onium_dpls(onium2, y, ext_dips2, n2)
            
            call get_ampltds(ext_dips1, n1, ext_dips2, n2, ampltd_evnt)

	    ! Add this event to overall results
	    ampltd(:,i_y) = ampltd(:,i_y) + ampltd_evnt(:)
         end do
         n_dip_emergency = ii + n_prev_events !DEBUG
         call periodic_output
         deallocate(ext_dips1, ext_dips2)
      end do 

      call writeall (n_prev_events+n_new_events)
                                                                        
!---------------------------------------------------------------------- 

contains 

subroutine read_all
      use sub_defs
      implicit none
      integer          i
      real(kind(1d0)) dummy
      

      call read_params_fdat(onm_size, maxy, n_prev_events, n_new_events)

      !!! ALLOCATE DIFFERENT OBJECTS HERE....
      allocate   (ampltd(0:data_io%n_pom, data_io%n_y))
      allocate   (ampltd_evnt(0:data_io%n_pom))
      write(0,*) 'Have allocated bins'
      if (n_prev_events == 0) then
	 ampltd = 0d0
      else
         do i = 1, data_io%n_y
            read(idat, *) dummy, ampltd(:, i)
         end do
	 ampltd(:,:) = ampltd(:,:) * n_prev_events
      endif

      !-- now that files are certified readable, make a backup copy of
      ! them in case the evolution to follow (or the output from it) go
      ! wrong. 
      call backup_files

end subroutine read_all

!**********************************************************************
!     SUBROUTINE WRITEALL: for use during loop and final output phase:
!     outputs parameters and results to specified devices
!
! GPS 29/11/94 (CCN3 95)
!**********************************************************************
subroutine writeall(local_n_events)
      integer, intent(in)  :: local_n_events
      integer i
      real(kind(1d0))  y

      call write_params_fdat(onm_size, maxy, local_n_events, iprm, idat)
      
      do i = 1, data_io%n_y
	 y  = maxy + (i - data_io%n_y) * data_io%y_int
         write(idat,20) 2d0*y, ampltd(:,i) / local_n_events
 20      format (25es13.5)                          
      end do                              

end subroutine writeall




!*************************************************************************
! Get the onium-onium interaction amplitude on a grid (covering the
! relevant interaction region) and then process it to obtain multiple
! pomeron amplitudes and the unitarised amplitude. (ampltd_evnt(0) =
! unitarised and ampltd_evnt(1:data_io%n_pom) = 1:n_pom pomerons.
!
! GPS 16/11/95 (CCN7 35)
!*************************************************************************
subroutine get_ampltds(ext_dips1, n1, ext_dips2, n2, ampltd_evnt)
      use gluons
      use constants
      use data_defs
      use sub_defs
      implicit none
      !----------------------------------------------------------------
      ! Arguments
      type(dpl_pntr), dimension(:), intent(in)  :: ext_dips1, ext_dips2
      integer,                      intent(in)  :: n1, n2
      real(kind(1d0)),             intent(out) :: ampltd_evnt(0:)
      !----------------------------------------------------------------
      ! Local variables
      integer, parameter  :: grid_size = 99
      integer                i
      real(kind(1d0))       gh_lo, gh_int, gv_lo, gv_int ! grid limits
      real(kind(1d0))       grid(0:grid_size, 0:grid_size) 
      real(kind(1d0))       grid_cp(0:grid_size, 0:grid_size) 
      real(kind(1d0))       grid_box_area
      !--------------------------------------------------------------------

      call onm_onm_grid(ext_dips1, n1, ext_dips2, n2, grid, gh_lo, gh_int,&
           & gv_lo, gv_int) 
      grid_box_area  = gv_int*gh_int
      grid_cp = grid

      ampltd_evnt(1) = grid_box_area * sum(grid_cp)
      ! Work out 1st to data_io%n_pom^th order contributions
      do i = 2, data_io%n_pom
         grid_cp = grid_cp * grid
         ampltd_evnt(i) = grid_box_area * sum(grid_cp) / fact(i)
      end do

      ! The unitarised cross section
      ampltd_evnt(0) = grid_box_area*((grid_size+1)**2-sum(exp(-grid)))

end subroutine get_ampltds

!----------------------------------------------------------------------
! Factorial function is needed, because n-pomeron exchange involves an n
! -factorial suppression
!----------------------------------------------------------------------
function fact(i)
      real(kind(1d0)) fact
      integer, intent(in) :: i
      integer             :: j 

      fact = 1d0
      do j = 1, i 
	 fact = fact * j      	 
      end do
end function fact




!======================================================================
! The following routines will probably be useful in the same form for
! nearly all driver routines for analysis of a single onium.
!======================================================================

!----------------------------------------------------------------------
!     Update the results files every bak_prd seconds
!----------------------------------------------------------------------
subroutine periodic_output
      integer, parameter   :: bak_prd = 60
      integer              :: n_bak = 1

      if (interval_elapsed(bak_prd)) then
         call writeall(n_prev_events+ii)
         write(iprm,*) 'Backup no ', n_bak,'of (estimated)', &
     &           int((dble(n_bak-1)*dble(n_new_events))/ii)+1
         write(iprm,*) 'Backups being done every',bak_prd,' seconds'
         write(iprm,*) '== every ', ii/n_bak,'evolutions'
         n_bak = n_bak + 1
      end if
end subroutine periodic_output

end program onm_onm_ftot





subroutine add_f0_v_c(ext_dips1, n1, ext_dips2, n2, r0, bins_v_c)
!      use bin_interface, except => add_f_to_bins
      use gluons
      use constants
      implicit none
      !----------------------------------------------------------------
      ! Arguments
      type(dpl_pntr), dimension(:), intent(in)    :: ext_dips1, ext_dips2
      integer,                      intent(in)    :: n1, n2
      real(kind(1d0)),              intent(in)    :: r0
      real(kind(1d0)),             intent(inout) :: bins_v_c(0:)
      !---- defines interaction parameters for a pair of dipoles ------
      type intrctn
	 complex(kind(1d0))  :: psn
	 real(kind(1d0))    :: r, sigma
      end type intrctn
      !---- Local variables -------------------------------------------
      type(intrctn)                             :: region
      integer      i1, i2, i, j, ic      ! counters
      complex(kind(1d0))   dipole_1(1:2), dipole_2(1:2)
      real(kind(1d0))       h_rgn_lo, h_rgn_hi, strength
      integer      iv_rgn_lo, iv_rgn_hi
      !
      !---- specifically for getting interaction at different r -----
      real(kind(1d0))     intrn
      complex(kind(1d0))  rpsns

      rpsns = cmplx(r0,0d0,kind(r0))
      intrn = 0d0
      strength = alpha_no_nc**2 / 2d0

      do i1 = 1, n1
	 do i2 = 1, n2
	    i = i1 + n1*(i2-1)   
	    ! Work out position and size of the region
	    ! vector to add to 2nd dipole such that its centre is at the
	    ! same point as the centre of the first dipole
            region%psn = (ext_dips1(i1)%p%hi_y_gluon%psn + &
                                   ext_dips1(i1)%p%lo_y_gluon%psn - &
                                   ext_dips2(i2)%p%hi_y_gluon%psn - &
                                   ext_dips2(i2)%p%lo_y_gluon%psn)/2.0d0

	    !----- The extent of the interaction region -------------------
            region%r = (ext_dips1(i1)%p%size + ext_dips2(i2)%p%size) * 2d0

            !----- points are taken along a line vertically up only ------
	    h_rgn_lo =  real(region%psn) - region%r
	    h_rgn_hi =  real(region%psn) + region%r
            if (h_rgn_lo > 0d0 .or. h_rgn_hi < 0d0) cycle

            !----- get bin on vertical line ------------------------------
            iv_rgn_lo = irad(aimag(region%psn) - region%r)
            iv_rgn_hi = irad(aimag(region%psn) + region%r)
            !----- the first condition should never occur...? -----------
            if (iv_rgn_lo > bn%nr .or. iv_rgn_hi < 0) cycle
	    
	    ! set up dipole positions in easily usable form (may waste some
	    ! time: but neater to program -- come back to it later?)
	    dipole_1(1) = ext_dips1(i1)%p%hi_y_gluon%psn
	    dipole_1(2) = ext_dips1(i1)%p%lo_y_gluon%psn
	    dipole_2(1) = ext_dips2(i2)%p%hi_y_gluon%psn
	    dipole_2(2) = ext_dips2(i2)%p%lo_y_gluon%psn

            iv_rgn_lo = max(0, iv_rgn_lo)
            iv_rgn_hi = min(bn%nr, iv_rgn_hi)


	    ! Add contributions from this interaction region to all
	    ! relevant radial points 
            intrn =  potential(dipole_1, dipole_2 + rpsns(j))
            ic = floor(log(ext_dips1(i1)%p%size) / dlogc)
            if (ic <= icmax) bins_v_c(ic) = bins_v_c(ic) + strength*intrn
	 end do
      end do

contains

!***************************************************************************
! Routine to work out the potential^2 between two dipoles with sources
! positioned at positions dipole_1 and dipole_2 -- without normalisation
! of (alpha_no_nc**2/ 2) -- NB interaction by two gluon exchange is 
! proportional to square of dipole dipole potential
!
! Equation # in write up?
!
! GPS 16/5/95 (CCN5 65)
!***************************************************************************
function potential(dipole_1, dipole_2)
     implicit none
     real(kind(1d0))      potential
     complex(kind(1d0)), intent(in)  :: dipole_1(1:2), dipole_2(1:2)
     real(kind(1d0))      numerator, denominator
     real(kind(1d0)),     parameter   :: max_potn = 1d3
     
     ! Work out square of absolute values of each quantity -- hopefully
     ! this is the fastest way of dealing with things. Requires factor of 4
     ! after the log though. 
     numerator    = abs_sqrd(dipole_2(2) - dipole_1(2)) * &
                    abs_sqrd(dipole_2(1) - dipole_1(1)) 
     denominator  = abs_sqrd(dipole_2(2) - dipole_1(1)) * &
                    abs_sqrd(dipole_2(1) - dipole_1(2))
     if (numerator == 0d0 .or. denominator == 0d0) then
        potential = max_potn
     else
        ! remember factor of 4 to take into account no sqrt in ratios
        potential = (log(numerator / denominator))**2 / 4d0 
     end if
     potential = min(potential, max_potn)
end function potential

! needed many times and much quicker than doing (abs())**2
function abs_sqrd(vector)
      real(kind(1d0)) abs_sqrd
      complex(kind(1d0)), intent(in) :: vector
      
      abs_sqrd = real(vector)**2 + aimag(vector)**2
end function abs_sqrd
          

end subroutine add_f_to_bins




