!**********************************************************************
!     Module that defines a data type to contain all info relating to
!     data that needs to be i/o-d
! GPS 11/12/94 (CCN3 113)
!**********************************************************************
module data_defs
      !-- see README for details
      type data_prms
	 double precision  :: y_wvfn, y_int
	 integer           :: n_y           
      end type data_prms

      type(data_prms) data_io
      data data_io /data_prms(0.0d0, 0.5d0, 5)/ 

end module data_defs
   

!***************************************************************************   
! Routine to determine the distribution of onium-onium scattering
! amplitudes at many values of r.
! The amplitudes are determined along a line going upwards -- both onia
! orintations are chosen randomly, with a linear sampling in r up to r = 1
! , and then a logarithmic sampling beyond -- the position within a given
! bin is chosen randomly.
!
! This should allow reconstruction of the full impact parameter
! distribution of the amplitude, unitarised or for any number of pomerons. 
!
! GPS 12/8/95 (CCN7 29)
!****************************************************************************
program onm_onm_bnd
      use emergency             !DEBUG
      use gluons                ! Definitions for gluons and dipoles 
      use cuts                  ! The cutoffs being used 
      use sub_defs              ! Some interface blocks 
      use iomodule              ! Open files and get command line args
      use data_defs             ! Def for data type containing data i/o info
      use binning          ! all the binning parameters
      use bin_interface    ! interface blks for all routines to do with bins
      use fbins_mdl        ! contains the array for the bins: fbins
      implicit none 

      !-----------------------------------------------------------------------
      type(dpl_pntr),   allocatable  :: ext_dips1(:), ext_dips2(:)
      !---- evolution details ----------------------------------------
      integer               :: n_prev_events, n_new_events
      double precision      :: onm_size, maxy, y, y_wvfn
      type(dipole), pointer :: onium1, onium2
      !---- 'internal' vars ------------------------------------------
      integer                ii, i_y
      integer                n1, n2
      integer       jseeda(2)
      !------------------------------------------------------------------------
  
      call read_all

      write(0,*) 'Am about to evolve'

      ! allow possibiliy of wavefunctions evolved to higher y than one
      ! will extract at the end.
      if (data_io%y_wvfn < maxy) then
         y_wvfn = maxy  
      else
         y_wvfn = data_io%y_wvfn
      end if
      
      do ii = 1,n_new_events
         !--- produce the wavefunctions for each onium -------------------
         call reset_counters

         call init_onium_rnd_phi(onium1, onm_size)
         call evolve_onium(onium1, y_wvfn, n1)
         allocate(ext_dips1(n1/2 + 1))

         call init_onium_rnd_phi(onium2, onm_size)
         call evolve_onium(onium2, y_wvfn, n2)
         allocate(ext_dips2(n2/2 + 1))
         
         !-- some trickery is needed to ensure that the same random radial
         ! positions get used when sampling the amplitude for a particular
         ! evolution of dipole configurations at different rapidities. 
         call hwrget(jseeda)
	 !--------------------------------------------------------------
	 ! For each value of y, extract the two sets of dipoles and then
	 ! work out the cross sections
         do i_y = 1, data_io%n_y
	    y  = maxy + (i_y - data_io%n_y) * data_io%y_int

            call extract_onium_dpls(onium1, y, ext_dips1, n1)
            call extract_onium_dpls(onium2, y, ext_dips2, n2)
            
            call hwrset(jseeda)
            call add_f_to_bins(ext_dips1, n1, ext_dips2, n2, fbins(:,:,i_y))

         end do
         n_dip_emergency = ii + n_prev_events !DEBUG
         call periodic_output
         deallocate(ext_dips1, ext_dips2)
      end do 

      call writeall (n_prev_events+n_new_events)
      deallocate(fbins)
                                                                        
!---------------------------------------------------------------------- 

contains 


!----------------------------------------------------------------------
! Read in all relevant data, allocating any matrices that are needed
!----------------------------------------------------------------------
subroutine read_all
      use sub_defs
      use bin_interface
      implicit none
      integer i

      !--- read in the parameters
      call read_params_ufdt(onm_size, maxy, n_prev_events, n_new_events)
      !--- allocate bins and read in (or init) the binned data
      call allc_read_fbins(data_io%n_y, n_prev_events)

      call backup_files
end subroutine read_all


!**********************************************************************
!     SUBROUTINE WRITEALL: for use during loop and final output phase:
!     outputs parameters and results to specified devices
!
! GPS 29/11/94 (CCN3 95)
!**********************************************************************
subroutine writeall(local_n_events)
      use bin_interface
      implicit none
      integer, intent(in)  :: local_n_events
      integer i

      call write_params_ufdt(onm_size, maxy, local_n_events)

      call write_fbins(local_n_events, iufdt)
end subroutine writeall


!----------------------------------------------------------------------
! Update the results files every bak_prd seconds
!----------------------------------------------------------------------
subroutine periodic_output
      integer, parameter   :: bak_prd = 60
      integer              :: n_bak = 1

      if (interval_elapsed(bak_prd)) then
         call writeall(n_prev_events+ii)
         write(iprm,*) 'Backup no ', n_bak,'of (estimated)', &
     &           int((dble(n_bak-1)*dble(n_new_events))/ii)+1
         write(iprm,*) 'Backups being done every',bak_prd,' seconds'
         write(iprm,*) '== every ', ii/n_bak,'evolutions'
         n_bak = n_bak + 1
      end if
end subroutine periodic_output
         

end program onm_onm_bnd

