!**********************************************************************
!     Module that defines a data type to contain all info relating to
!     data that needs to be i/o-d
! GPS 11/12/94 (CCN3 113)
!**********************************************************************
module data_defs
      !-- see README for details of type defn
      type data_prms
         double precision  rmin, rmax
         integer nr
         double precision  c_hwdth
         integer nc
      end type data_prms
      
      type(data_prms)        data_io
      data data_io /data_prms(1d0,1d4,80,1.5d0,13)/
end module data_defs


!**********************************************************************    
!     Routine to determine spatial distribution of dipoles for various
!     dipole sizes
!
! GPS 20/2/95 CCN4(103) + 4/11/95 (CCN8 9)
!********************************************************************** 
program spatial_wf
      use emergency             !DEBUG
      use gluons                ! Definitions for gluons and dipoles 
      use cuts                  ! The cutoffs being used 
      use sub_defs              ! Some interface blocks 
      use iomodule              ! Open files and get command line args
      use data_defs             ! Def for data type containing data i/o info
      implicit none 
      !--- variables which are always needed --------------------------
      type(dpl_pntr), allocatable   :: extracted_dipoles(:)
      type(dipole),   pointer       :: onium
      double precision              :: onm_size, maxy
      integer                ii, n
      integer                n_new_events, n_prev_events
      !--- vars specific to this routine -------------------------------
      double precision, allocatable :: space_bins(:,:)
      double precision              :: log_rint, log_cwdth
      !------------------------------------------------------------------

      call read_all

      log_rint  = log(data_io%rmax/data_io%rmin) / data_io%nr
      log_cwdth = 2d0 * log(data_io%c_hwdth)

      write(0,*) 'Evolution starting'

      do ii = 1,n_new_events
         call reset_counters
         call init_onium_rnd_phi(onium, onm_size)
         call evolve_onium(onium, maxy, n)
         allocate(extracted_dipoles(n/2 + 1))
         call extract_onium_dpls(onium, maxy, extracted_dipoles, n)

         !-- bin the positions and sizes of the dipoles
         call add_to_bins(space_bins,extracted_dipoles,n) 

         !-- output contents of the bins every 60 seconds.
         call periodic_output
         n_dip_emergency = ii + n_prev_events !DEBUG
         deallocate(extracted_dipoles)
      end do 

      !-- output the data for the final time
      call writeall (n_prev_events+n_new_events)
                                                                        
!---------------------------------------------------------------------- 

contains 

!========================================================================
! Various routines specific to the task of analysing spatial distribution
! of dipoles of various sizes in an evolved onium wavefunction.
!======================================================================


!----------------------------------------------------------------------
! Read in all relevant data, allocating any matrices that are needed
!----------------------------------------------------------------------
subroutine read_all
      use sub_defs
      implicit none
      double precision       dummy
      integer                i

      call read_params_fdat(onm_size, maxy, n_prev_events, n_new_events)

      allocate   (space_bins(data_io%nr, data_io%nc))
      write(0,*) 'Have allocated bins'
      if (n_prev_events == 0) then
         space_bins = 0.0d0 
      else
         do i = 1, data_io%nr
            read(idat, *) dummy, space_bins(i,1:data_io%nc)
         end do
         space_bins = space_bins * n_prev_events
      endif
      call backup_files

end subroutine read_all




!----------------------------------------------------------------------
!     SUBROUTINE WRITEALL: for use during loop and final output phase:
!     outputs parameters and results to specified devices
!
! GPS 29/11/94 (CCN3 95)
!----------------------------------------------------------------------
subroutine writeall(local_n_events)
      use sub_defs
      use iomodule
      implicit none
      integer, intent(in)  :: local_n_events
      integer i

!      call write_params_fdat(onm_size, maxy, local_n_events, iprm, idat)
      ! iprm and idat are now optional args
      call write_params_fdat(onm_size, maxy, local_n_events)
      do i=1,data_io%nr 
         write(idat,20) exp((i-1)*log_rint), &
     &        space_bins(i,1:data_io%nc)/(local_n_events)
 20      format(f14.3,20e14.5)
      enddo 
end subroutine writeall


!----------------------------------------------------------------------
!     Add to distribution of number of dipoles of a particular size
!     occuring in an onium
!                                                                       
! GPS 10/01/95 CCN3 164 
!----------------------------------------------------------------------
subroutine add_to_bins(space_bins,extracted_dipoles,n) 
      use cuts 
      use gluons 
      implicit none 
      double precision,         dimension(:,:), intent(inout) :: space_bins 
      type(dpl_pntr), dimension(:), intent(in)      :: extracted_dipoles 
      integer,                      intent(in)      :: n 
! Local variables                                                       
      integer ir, ic , j
      double precision  c_bin, r
      !double precision  size, scale  !! debug
                   
      do j = 1, n
         ! Only accept dipoles if they fall into a bin
         ! Take bin vals to be in middle of bin
         ic = floor(log(extracted_dipoles(j)%p%size/data_io%rmin)&
              & / log_cwdth + 0.5d0) + 1
         if (ic < 1 .or. ic > data_io%nc) cycle  ! Don't bin this  dipole
         c_bin = data_io%rmin * exp((ic-1) * log_cwdth)
         ! Distances are measured from centres of dipoles
         ! bin vals are from low edge of bin.
         r  = abs(extracted_dipoles(j)%p%lo_y_gluon%psn + &
     &            extracted_dipoles(j)%p%hi_y_gluon%psn) / 2d0
         r  = r / sqrt(c_bin)
         if (r == 0d0) cycle                    ! avoid overflows
         ir = floor(log(r) / log_rint) + 1       
         if (ir < 1 .or. ir > data_io%nr) cycle ! Don't bin this dipole
         space_bins(ir, ic) = space_bins(ir, ic) + 1d0
      end do

end subroutine add_to_bins


!======================================================================
! The following routines will probably be useful in the same form for
! nearly all driver routines for analysis of a single onium.
!======================================================================
!----------------------------------------------------------------------
! Update the results files every bak_prd seconds
!----------------------------------------------------------------------
subroutine periodic_output
      integer, parameter   :: bak_prd = 60
      integer              :: n_bak = 1

      if (interval_elapsed(bak_prd)) then
         call writeall(n_prev_events+ii)
         write(iprm,*) 'Backup no ', n_bak,'of (estimated)', &
     &           int((dble(n_bak-1)*dble(n_new_events))/ii)+1
         write(iprm,*) 'Backups being done every',bak_prd,' seconds'
         write(iprm,*) '== every ', ii/n_bak,'evolutions'
         n_bak = n_bak + 1
      end if
end subroutine periodic_output


end program spatial_wf



	 


